#' @name addprocessnoise
#' @aliases addprocessnoise 
#' @title addprocessnoise
#' @description function to adds process noise to the deterministic model input
#' @param function_handle model function that takes a vector (x) as input and returns an output vector
#' @param x a vector at which the function_handle is being evaluated
#' @param Qmod A scalar/ vector/ array with process (co)variances. If scalar, the value is multiplied by an identity matrix with dimensions equal to the length of the vector x. If vector, the values form the diagonal of the covariance matrix with off-diagonal elements set to zero. The array is directly used without any further processing. 
#' @param seedval A scalar value to be used as a random seed for generating process noise
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return Model output vector with process noise added to it 
#' @references %% ~put references to the literature/web site here ~
#' @author Riddhi Singh
#' @note %% add value
#' @seealso %%  ~~further notes~~
#' @examples  
#' addprocessnoise(f_model, c(1e5, 6000, 2500), 10, 100)
#' @keywords NONE
#' @export

addprocessnoise <-
  function(function_handle, Qmod, qtype, seedval, x, theta, timestep=0.01, t=0)
  {
    ln           <- length(x) 
    Q            <- getQ(Qmod, ln)      
    testmat      <- array(0, dim=c(ln,ln))
    if (identical(Q,testmat)==FALSE) {
      L          <- chol(Q)    # since Q is a covariance we can use Cholesky decomp    
      set.seed(seedval)
      if (qtype == 0) {
      w            <- L%*%rnorm(ln,0,1)       
      } else {        
        w          <- exp(rnorm(ln,0,L))       
      }
    } else {
      w          <- rep(0, ln)
    }         
    if (qtype == 0) {
      return(function_handle(x, theta, timestep, t)+w)    
    } else {
      return(function_handle(x, theta, timestep, t)*w)    
    }
    #cleanup
    rm(Q, testmat, L, w)
  }