#' @name basecasesetup
#' @aliases basecasesetup
#' @title Problem set up for the base case model
#' @description This function generates the synthetic truth and observations for the base case model
#' @param input  A list of user defined or default values for model
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @return  A list with the synthetic truth and observations
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note      %% add value
#' @seealso   \code{\link{setnltsdefault}}
#' @examples
#' #using the demostration example set up
#' input              <- setnltsdefault()
#' setup              <- nltssetup(input)
#' @export

basecasesetup <-
  function(input)
  {
    measurement_handle <- input$measurement_handle
    model_handle       <- input$model_handle
    # setting up variables related to the synthetic truth
    timerange <- seq(1,input$tend, by = input$timestep)                               # define the timerange to run the model
    N         <- length(timerange)                                                    # total number of time steps
    numpars   <- input$numpars    
    numstates <- input$numstates
    numvars   <- numpars + numstates
    z         <- rep(NA, N)
    # Calculate "true" state vector over time from dynamics.
    xtrue      <- array(NA,dim=c(numvars,N))
    xtrue[1,1] <- input$xtruestate
    numfor     <- length(measurement_handle(input$xtruestate))
    # when actual measurements are available, they can be directly assigned
    # otherwise the model is run with assumed 'true' values
    for (i in 1:N) {
      if (i==1) {
        xtrue[1:numstates,i]   <- addprocessnoise(model_handle,input$Qmod[1:numstates],input$qtype,input$seed[2]+i,xtrue[1,1],input$xtruepars,input$timestep,i)   + input$Bmat*input$uinput[i]        
      } else {
        xtrue[1,i]   <- addprocessnoise(model_handle,input$Qmod[1:numstates],input$qtype,input$seed[2]+i,xtrue[1,i-1],input$xtruepars,input$timestep,i) + input$Bmat*input$uinput[i]
      }
      tempid <- numstates+1
      xtrue[tempid:numvars,i] <- input$xtruepars
    }    
    # Create noisy scalar measurements, these measurements are used in the filter:
    # note here that first measurement is at t=1
    set.seed(input$seed[1])
    for (i in 1:N) {
      # since measurement of phosphorus cannot be negative, set any negative values to 0
      z[i]             <- max(0, measurement_handle(xtrue[,i])+sqrt(input$R)*rnorm(n = numfor, mean = 0, sd = 1))  # add noise to scalar meas
    }
    #define the output
    setup           <- list()
    setup$xtrue     <- xtrue
    setup$z         <- z    
    return(setup)
  }
