#' @name f_lakestate
#' @aliases f_lakestate
#' @title the lake model
#' @description Model of the lake to update the phosphorus fluxes at every time step from Carpenter et al. 1999. The model is set up to update the lake's state variable (Phosphorus in the lake).
#' @param x input to the model: phosphorus at previous time step
#' @param theta b and q parameter values at which to estimate the next state
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  ouput phosphorus values at the next time step
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso \code{\link{lakesetup}} \code{\link{setlakedefault}}
#' @examples
#' x  <- c(0.5, c(0.4,2))
#' y  <- f_lakestate(x)
#' @export

f_lakestate <-
  function(x, theta, timestep = 1, t = 0)
  {    
    ln           <- length(x)
    xp           <- rep(NA, ln)    
    if ((is.nan(x[1])==TRUE) || (is.nan(theta[1])==TRUE) || (is.nan(theta[2])==TRUE) || (is.na(x[1])==TRUE) || (is.na(theta[1])==TRUE) || (is.na(theta[2])==TRUE)) {
      xp[1]      <- NaN 
    } else {
      # the original model equation
      xp[1]      <- x[1]*(1-theta[1]) + x[1]^theta[2]/(1+x[1]^theta[2])
    }
    return(xp)
    #cleanup
    rm(x,ln,xp)
  }