#' @name lakeparscheck
#' @aliases lakeparscheck
#' @title Checks if given matrix satisfies ranges of parameters for the lake model
#' @description This function checks for unrealistic values of the lake model parameters and returns realistic sets.
#' @param x_enspars A matrix of parameters of size numpars x N, where N is the number of sets and numpars are number of lake model parameters
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return indices The locations of points to be removed
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso  \code{\link{ensemblekalmanfilter}}, \code{\link{particlefilter}}
#' @examples
#' indices          <- lakeparscheck(x_enspars)
#' @export

lakeparscheck <-
  function(x_enspars)
  {
    numpars    <- dim(x_enspars)[1]
    # remove unrealistic parameter estimates
    for (i in 1:numpars) {
      # no parameter can be negative
      if (i==1) {
        temp1             <- which(x_enspars[i,]<0)
      } else {
        temp1             <- c(temp1, which(x_enspars[i,]<0))
      }
      # b parameter cannot be greater than 1
      if (i==1) {
        temp2             <- which(x_enspars[i,]>1)
      } else {
        # q parameter has to be between 2 and 8
        temp3             <- which(x_enspars[i,]<2)
        temp4             <- which(x_enspars[i,]>8)
      }
    }
    indices                  <- union(union(union(temp1,temp2),temp3),temp4)
    return(indices)
  }
