#' @name lakestatecheck
#' @aliases lakestatecheck
#' @title Checks if given matrix satisfies ranges of state for the lake model
#' @description This function checks for unrealistic values of the lake model states and returns realistic states.
#' @param x_ensstate A matrix of states of size numstates x N, where N is the number of sets and numstates are number of lake model states
#' @param limits A matrix of lower and upper limits of each state, size 2xnumstates 
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  Updated x_ensstate
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso  \code{\link{ensemblekalmanfilter}}, \code{\link{particlefilter}}
#' @examples
#' x_ensstate          <- lakestatescheck(x_ensstate)
#' @export

lakestatecheck <-
  function(x_ensstate)
  {
    numstates    <- dim(x_ensstate)[1]
    # remove unrealistic state estimates
    for (i in 1:numstates) {
      # no state can be negative
      if (i==1) {
        temp             <- which(x_ensstate[i,]<0)
      } else {
        temp             <- union(temp, which(x_ensstate[i,]<0))
      }
    }
    return(temp)
  }