#' @name likelihood_update
#' @aliases likelihood_update 
#' @title Update particle weights based on the likelihood calculated using the simulations with observations
#' @description This measurement function assumes that the radar is located
#' @param xforPF An array of dimension n_ens x numfor with the forecasted variable values for each particle
#' @param par_w A vector of particle weights
#' @param observations  A scalar with the observation at the time of forecast
#' @param R Measurement variance                   
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @return  Updated particle weights
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @seealso dafilter.particle
#' @examples %%  ~~any examples~~

likelihood_update <-
function(xfor,observations,R) {
  ln         <- length(xfor)   
  lkhd       <- rep(NA,ln)
  for (par in 1:ln) {
    # Generate the weights for each of the particles
    # The weights are based upon the probability of the given
    # observation for a particle, GIVEN the synthetic observatison
        
    # The observation error is assumed to be normal with variance R. 
    # The likelihood of the particle given the measurement z[time] is simply
    # the probability distribution function of the normal distribution with variance R
    # returning the log likelihood to keep track of small probabilities
    lkhd[par] <- dnorm(xfor[,par],observations,sqrt(R),log=TRUE)    
 } # end loop over particles 
 return(lkhd)
}