#' @name likelihoodscans
#' @aliases likelihoodscans
#' @title Likelihood as a function of lake parameters
#' @description Plots the likelihood function as a function of b and q parameters - this helps in diagnosing performance of MCMC and the particle filter
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value

likelihoodscan <- 
  function() {    
    # set ranges of initial phosphorus, b, and q
    # note: if input$prior = 1, the prior is lognormal, it cannot start from initp=0
    initp           <- c(0.0)
    #     initp           <- 0
    bsamples        <- seq(0.1,0.8, by = 0.01)
    qsamples        <- seq(1,    8, by = 0.1)    
    lnp             <- length(initp)    
    lnb             <- length(bsamples)
    lnq             <- length(qsamples)
    
    # define end year
    endyr           <- 99
    # initialize the likelihood vector
    lkhd            <- array(NA, c(lnp, lnb, lnq))
    pr              <- array(NA, c(lnp, lnb, lnq))
    pst             <- array(NA, c(lnp, lnb, lnq))
    # set up the lake problem
    input          <- setlakedefault()
    setup          <- lakesetup(input)
    # generate likelihood function for each b and q sample
    for (ps in 1 : lnp) {
      for (bs in 1 : lnb) {
        for (qs in 1 : lnq) {     
          puse           <- initp[ps]
          buse           <- bsamples[bs]
          quse           <- qsamples[qs]          
          beta           <- c(puse, buse, quse)    
          # set option = 1 for likelihood output only, opiton = 2 for prior only
          lkhd[ps,bs,qs] <- logposteriorlake(beta, input, setup$z, endyr, 1)
          pr[ps,bs,qs]   <- logposteriorlake(beta, input, setup$z, endyr, 2)
          pst[ps,bs,qs]  <- logposteriorlake(beta, input, setup$z, endyr, 0)
        }
      }      
    }
    
    
    #     myColorRamp <- function(colors, values) {
    #       v <- (values - min(values))/diff(range(values))
    #       x <- colorRamp(colors)(v)
    #       rgb(x[,1], x[,2], x[,3], maxColorValue = 255)
    #     }
    #     cols    <- myColorRamp(c("red", "orange","yellow","green","blue"), lkhd[ch,,])         
    #   # plot the likelihood  
    # choose the initial phosphorus pollution at which to visualize the likelihood
    ch     <-  c(1,3,6)
    for (vl in 1:3) {
      for (i in 1:3) {
        if (vl == 1) {
          # plotting likelihood
          if (i==1) {  cairo_pdf(paste("Lkhdb1_8_init0_", endyr, ".pdf", sep=""), width=10, height=6)    
          } else if (i==2) {cairo_pdf(paste("Lkhdb1_8_init2_", endyr, ".pdf", sep=""), width=10, height=6)    
          } else if (i==3) {cairo_pdf(paste("Lkhdb1_8_init5_", endyr, ".pdf", sep=""), width=10, height=6)    
          }
        } else if (vl == 2) {
          # plotting prior
          if (i==1) {  cairo_pdf(paste("prb1_8_init0_", endyr, ".pdf", sep=""), width=10, height=6)    
          } else if (i==2) {cairo_pdf(paste("prb1_8_init2_", endyr, ".pdf", sep=""), width=10, height=6)    
          } else if (i==3) {cairo_pdf(paste("prb1_8_init5_", endyr, ".pdf", sep=""), width=10, height=6)    
          }
        } else if (vl==3) {
          # plotting posterior
          if (i==1) {  cairo_pdf(paste("pstb1_8_init0_", endyr, ".pdf", sep=""), width=10, height=6)    
          } else if (i==2) {cairo_pdf(paste("pstb1_8_init2_", endyr, ".pdf", sep=""), width=10, height=6)    
          } else if (i==3) {cairo_pdf(paste("pstb1_8_init5_", endyr, ".pdf", sep=""), width=10, height=6)    
          }
        }      
        par(bty="n")
        #       par(new=TRUE)
        # plot the likelihood scan
        if (vl == 1) {
          filled.contour(bsamples, qsamples, lkhd[ch[i],,], color.palette=terrain.colors, nlevels = 40, 
                         xlab = paste('b parameter [-]',sep=""), ylab = paste('q parameter [-]',sep=""), 
                         cex.lab = 1.5,cex.axis = 1.5,
                         main = paste('Log likelihood at ', endyr, ' yrs, initial P = ', initp[ch[i]], sep=""),
                         plot.axes = {points(input$xtrueinit[2], input$xtrueinit[3], pch=8) 
                                      axis(1, seq(0.1,0.8, by = 0.1)) 
                                      axis(2, seq(1, 8, by = 1))},
                         key.title = title("Log\nlikelihood [-]", cex.main=0.8, line=0.5))          
        } else if (vl==2) {
          filled.contour(bsamples, qsamples, pr[ch[i],,], color.palette=terrain.colors, nlevels = 40, 
                         xlab = paste('b parameter [-]',sep=""), ylab = paste('q parameter [-]',sep=""), 
                         cex.lab = 1.5,cex.axis = 1.5,
                         main = paste('Log prior at ', endyr, ' yrs, initial P = ', initp[ch[i]], sep=""),
                         plot.axes = {points(input$xtrueinit[2], input$xtrueinit[3], pch=8) 
                                      axis(1, seq(0.1,0.8, by = 0.1)) 
                                      axis(2, seq(1, 8, by = 1))},
                         key.title = title("Log\nprior [-]", cex.main=0.8, line=0.5))    
        } else if (vl==3) {
          filled.contour(bsamples, qsamples, pst[ch[i],,], color.palette=terrain.colors, nlevels = 40, 
                         xlab = paste('b parameter [-]',sep=""), ylab = paste('q parameter [-]',sep=""), 
                         cex.lab = 1.5,cex.axis = 1.5,
                         main = paste('Log posterior at ', endyr, ' yrs, initial P = ', initp[ch[i]], sep=""),
                         plot.axes = {points(input$xtrueinit[2], input$xtrueinit[3], pch=8) 
                                      axis(1, seq(0.1,0.8, by = 0.1)) 
                                      axis(2, seq(1, 8, by = 1))},
                         key.title = title("Log\nposterior [-]", cex.main=0.8, line=0.5))             
        }
        dev.off()  
      }
    }      
  }