#' @name logpriorbasecase
#' @aliases logpriorbasecase
#' @title The log of the prior for the linear model
#' @description This function estimates the prior, to be used for MCMC analysis
#' @param beta The list of parameters that MCMC needs to identify
#' @param input Any additional inputs for the lake model, same as the output from setbasecasedefault()
#' @param yobs The vector of observations
#' @return  log of the posetrior (unnormalized)
#' @author Riddhi Singh
#' @importFrom mvtnorm dmvnorm

logpriorbasecase <-
  function(beta,priornext,priorchoice) {
    # estimate the priors
    if (priorchoice==0) {        
      logprior  <- dmvnorm(beta,priornext$meanx,priornext$varx,log=TRUE)        
    } else {
      numvars   <- length(beta)
      location  <- rep(NA, numvars)
      scale     <- rep(NA, numvars)
      for (i in 1:numvars) {
        tempmean        <- priornext$meanx[i]
        tempvar         <- priornext$varx[i]
        location[i]     <- log(tempmean/sqrt(1+tempvar/tempmean^2))
        scale[i]        <- sqrt(log(1+tempvar/tempmean^2))
      }      
      scale     <- diag(scale)         
      logprior  <- dmvnorm(log(as.vector(beta)), location, scale, log=TRUE)   
      if (is.nan(logprior)==TRUE) { logprior = -1e9} 
    }
    return(logprior)
  }