#' @name paperplot1
#' @aliases paperplot1
#' @title Runs the code for the generating Figure 1 in the publication.
#' @description This function runs the code for generating Figure 1 in the study.  
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot1 <- 
  function() {
    # clean up
    rm(list=ls())    
    # make the plot
    cairo_pdf(paste("Figure1.pdf", sep=""), width=8, height=4)
    # colors for bterm, qterm and input term, respectively
    cols      <- c(rgb(0,0,0), rgb(0.5,0.5,0.5), rgb(0.5,0.5,0.5))
    pchvals   <- c(1,1,0)
    ltyvals   <- c(1,1,2)
    lwdvals   <- c(2,2,2)
    #start plotting
    layout(matrix(c(1,2,3,3), 2, 2, byrow = TRUE))
    par(mgp=c(1.8,0.5,0))
    par(mar=c(3,3,2,1))    
    # Plot a. two stable equilibria
    input       <- setlakedefault(c(1,1),0)    
    # for the state space
    xvals       <- seq(0,2.5,by=1e-5)
    bterm       <- rep(NA,length(xvals))
    qterm       <- rep(NA,length(xvals))
    ainput      <- rep(0.01,length(xvals))
    for (i in 1: length(xvals)) {
      bterm[i]  <- xvals[i]*input$xtruepars[1]
      qterm[i]  <- xvals[i]^input$xtruepars[2]/(xvals[i]^input$xtruepars[2]+1)
    }
    inputs      <- ainput + qterm
    diffterm    <- bterm - inputs
    temp        <- sort(abs(diffterm),index.return=TRUE)
    equilib     <- sort(temp$ix[1:3])
    pterm       <- xvals
    # plot
    plot(pterm,bterm,type="l",col=cols[1],xlim=range(pterm),ylim=range(c(bterm,inputs)),
         lty=ltyvals[1],lwd=lwdvals[1],xlab='Phosphorus in the lake [-]',ylab='Rates of flux [-]',cex.lab=1.2)
    lines(pterm,inputs,pch=pchvals[2],col=cols[2],lty=ltyvals[2],lwd=lwdvals[2]) 
    lines(pterm,ainput,pch=pchvals[3],col=cols[3],lty=ltyvals[3],lwd=lwdvals[3])
    for (i in 1:length(equilib)) {
      if (i!=2) {  
        points(pterm[equilib[i]],inputs[equilib[i]],bg=rgb(0.7,0.7,0.7),col=rgb(0,0,0),pch=21,cex=1.5) 
      } else {
        points(pterm[equilib[i]],inputs[equilib[i]],bg=rgb(0,0,0,0),col=rgb(0,0,0),pch=21,cex=1.5,lwd=2) 
      }
    }
    mtext('a. Two stable equilibria',side=3,line=0.5,adj=0,cex=1.2)
    #add legend
    legtext <- c('Sink','Source:recyle','Source:input<0.06')
    legend(-0.10, 1.15, legtext, box.col=NA,bg=NA,lty=ltyvals,
           pch=c(NA,NA,NA),col=cols,lwd=lwdvals,pt.cex=c(2,2,2))
    
    # only one stable equilibrium
    input       <- setlakedefault(c(1,1),0)  
    # for the state space
    xvals       <- seq(0,2.5,by=1e-5)
    bterm       <- rep(NA,length(xvals))
    qterm       <- rep(NA,length(xvals))
    ainput      <- rep(0.060,length(xvals))
    for (i in 1: length(xvals)) {
      bterm[i]  <- xvals[i]*input$xtruepars[1]
      qterm[i]  <- xvals[i]^input$xtruepars[2]/(xvals[i]^input$xtruepars[2]+1)
    }
    inputs      <- ainput + qterm
    diffterm    <- bterm - inputs
    temp        <- sort(abs(diffterm),index.return=TRUE)
    equilib     <- sort(temp$ix[1])
    pterm       <- xvals
    # plot
    plot(pterm,bterm,type="l",col=cols[1],xlim=range(pterm),ylim=range(c(bterm,inputs)),
         lty=ltyvals[1],lwd=lwdvals[1],xlab='Phosphorus in the lake [-]',ylab='Rates of flux [-]',cex.lab=1.2)
    lines(pterm,inputs,pch=pchvals[2],col=cols[2],lty=ltyvals[2],lwd=lwdvals[2]) 
    lines(pterm,ainput,pch=pchvals[3],col=cols[3],lty=ltyvals[3],lwd=lwdvals[3])
    for (i in 1:length(equilib)) {
      if (i!=2) {  
        points(pterm[equilib[i]],inputs[equilib[i]],bg=rgb(0.7,0.7,0.7),col=rgb(0,0,0),pch=21,cex=1.5) 
      } else {
        points(pterm[equilib[i]],inputs[equilib[i]],bg=rgb(0,0,0),col=rgb(0,0,0),pch=21,cex=1.5) 
      }
    }
    mtext('b. One stable equilibrium',side=3,line=0.5,adj=0,cex=1.2)
    #add legend
    legtext <- c('Sink','Source:recyle','Source:input>=0.06')
    legend(-0.10,1.15,legtext,box.col=NA,bg=NA,lty=ltyvals,
           pch=c(NA,NA,NA),col=cols,lwd=lwdvals,pt.cex=c(2,2,2))
      
    # input policy plots
    input       <- setlakedefault(c(1,1),0)
    pol1        <- input$uinput
    input       <- setlakedefault(c(1,1),1)
    pol2        <- input$uinput
    plot(pol2, type="l", lwd=2, lty=2,
         ylab="Input pollution [-]", xlab='Time [years]', cex.lab=1.2)
    lines(pol1, lty=1, lwd=2)
    mtext('c. Pollution policies tested', side=3, line=0.5, adj=0, cex=1.2)
    legtext <- c('Low emissions','High emissions')
    legend(0.01, 0.20, legtext, box.col=NA,bg=NA,lty=c(1,2),
           pch=c(NA,NA,NA),col=rgb(0,0,0),lwd=c(2,2))
    
    dev.off()
#     # for the oligotrophic lake
#     input       <- setlakedefault(c(1,1),0)    
#     outvars     <- array(NA, c(length(input$xtrueinit), input$tend))
#     bterm       <- rep(NA, input$tend)
#     qterm       <- rep(NA, input$tend)    
#     ainput      <- input$uinput
#     outvars[,1] <- input$xtrueinit
#     runtime     <- input$tend-1
#     for (i in 1:runtime) {    
#       bterm[i]       <- input$xtrueinit[2]*outvars[,i]
#       qterm[i]       <- (outvars[,i]^input$xtrueinit[3]/(1+outvars[,i]^input$xtrueinit[3]))
#       outvars[,i+1]  <- outvars[,i] - bterm[i] + qterm[i]
#       outvars[1,i+1] <- outvars[1,i+1] + ainput[i]      
#     }
#     pterm               <- outvars[1,]
#     bterm[input$tend]   <- input$xtrueinit[2]*pterm[input$tend]
#     qterm[input$tend]   <- pterm[input$tend]^input$xtrueinit[3]/(1+pterm[input$tend]^input$xtrueinit[3])
#            
#     # plot
#     plot(pterm, bterm, type="l",col=cols[1], xlim=range(pterm), ylim=range(c(qterm, bterm)),
#          xlab='Phosphorus in the lake [-]',ylab='Rates of flux [-]',cex.lab=1.5)
#     points(pterm,  bterm, pch=pchvals[1], col=cols[1])
#     lines(pterm,   qterm+ainput, pch=pchvals[2], col=cols[2]) 
#     points(pterm,  qterm+ainput, pch=pchvals[2], col=cols[2])      
#     lines(pterm,  ainput, pch=pchvals[3], col=cols[3])
#     points(pterm, ainput, pch=pchvals[3], col=cols[3])
#     
#     #add legend
#     legtext <- c('Sinks','Source-recyle','Source-input')
#     legend(0.01, 0.80, legtext, box.col='black',lty=c(1,1,1),pch=pchvals,col=cols,lwd=c(2,2,2),pt.cex=c(2,2,2))
#     
#     # for the eutorphic lake    
#     input       <- setlakedefault(c(1,1),1)    
#     outvars     <- array(NA, c(length(input$xtrueinit), input$tend))
#     bterm       <- rep(NA, input$tend)
#     qterm       <- rep(NA, input$tend)    
#     ainput      <- input$uinput
#     outvars[,1] <- input$xtrueinit
#     runtime     <- input$tend-1
#     for (i in 1:runtime) {    
#       bterm[i]       <- input$xtrueinit[2]*outvars[,i]
#       qterm[i]       <- (outvars[,i]^input$xtrueinit[3])/(1+outvars[,i]^input$xtrueinit[3])
#       outvars[,i+1]  <- outvars[,i] - bterm[i] + qterm[i]
#       outvars[1,i+1] <- outvars[1,i+1] + ainput[i]      
#     }
#     pterm               <- outvars[1,]
#     bterm[input$tend]   <- input$xtrueinit[2]*pterm[input$tend]
#     qterm[input$tend]   <- pterm[input$tend]^input$xtrueinit[3]/(1+pterm[input$tend]^input$xtrueinit[3])
#     
#     # plot
#     plot(pterm, bterm, type="l",col=cols[1], xlim=range(pterm), ylim=range(c(qterm, bterm)),
#          xlab='Phosphorus in the lake [-]',ylab='Rates of flux [-]',cex.lab=1.5)
#     points(pterm,  bterm, pch=pchvals[1], col=cols[1])
#     lines(pterm,   qterm+ainput, pch=pchvals[2], col=cols[2]) 
#     points(pterm,  qterm+ainput, pch=pchvals[2], col=cols[2])      
#     lines(pterm,  ainput, pch=pchvals[3], col=cols[3])
#     points(pterm, ainput, pch=pchvals[3], col=cols[3])
#     
#     #add legend    
#     legend(0.1, 1.0, legtext, box.col='black',lty=c(1,1,1),pch=pchvals,col=cols,lwd=c(2,2,2),pt.cex=c(2,2,2))
#         dev.off()
  }