#' @name paperplot2
#' @aliases paperplot2
#' @title Runs the code for generating the figures in the publication
#' @description Plots the likelihood function as a function of b and q parameters.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot3 <- 
  function(option=0,endyear=100,alldata=NULL) {    
    # set up the lake problem
    input           <- setlakedefault(c(1,1),option)
    setup           <- lakesetup(input)
    logposteriormod <- input$logposteriormod    
    # set ranges of initial phosphorus, b, and q
    brange          <- c(0.1,0.8)
    qrange          <- c(2,8)
    initp           <- c(0.0)
    bsamples        <- seq(brange[1],brange[2],by=0.01)
    qsamples        <- seq(qrange[1],qrange[2],by=0.1)        
    lnp             <- length(initp)    
    lnb             <- length(bsamples)
    lnq             <- length(qsamples)    
    # define end year, should be input$tend for likelihood at t=100
    if (is.null(alldata)==TRUE) {
      # initialize the likelihood vector
      lkhd            <- array(NA,c(lnp,lnb,lnq))
      pr              <- array(NA,c(lnp,lnb,lnq))
      pst             <- array(NA,c(lnp,lnb,lnq))
      # generate likelihood function for each b and q sample
      for (ps in 1 : lnp) {
        for (bs in 1 : lnb) {
          for (qs in 1 : lnq) {     
            puse           <- initp[ps]
            buse           <- bsamples[bs]
            quse           <- qsamples[qs]          
            beta           <- c(puse,buse,quse)   
            # set option = 1 for likelihood output only, opiton = 2 for prior only
            lkhd[ps,bs,qs] <- logposteriormod(beta,input,setup$z,xstate=NULL,begintime=1,endtime=endyear,option=1,priornext=NULL)
            pr[ps,bs,qs]   <- logposteriormod(beta,input,setup$z,xstate=NULL,begintime=1,endtime=endyear,option=2,priornext=NULL)
            pst[ps,bs,qs]  <- logposteriormod(beta,input,setup$z,xstate=NULL,begintime=1,endtime=endyear,option=0,priornext=NULL)
          }
        }      
      }    
      alldata              <- list(pst=pst,lkhd=lkhd,pr=pr)
      if (option==0) {
        cairo_pdf(paste("Figure3a_",endyear,".pdf",sep=""),width=10,height=6)  
        titlenm <- '(a) Low emissions'
        fname   <- paste("paperplot3a_",endyear,".RData",sep="")
        save(alldata,file=fname)
      } else if (option==1) {
        cairo_pdf(paste("Figure3b_",endyear,".pdf",sep=""),width=10,height=6)  
        titlenm <- '(b) High emissions'
        fname   <- paste("paperplot3b_",endyear,".RData",sep="")
        save(alldata,file=fname)
      } 
    } else {
      lkhd    <- alldata$lkhd
      # now plot the results 
      if (option==0) {
        cairo_pdf(paste("Figure3a_",endyear,".pdf",sep=""),width=10,height=6)  
        titlenm <- '(a) Low emissions'
      } else if (option==1) {
        cairo_pdf(paste("Figure3b_",endyear,".pdf",sep=""),width=10,height=6)  
        titlenm <- '(b) High emissions'
      }
    }
    # plotting relative likelihoods
    newlkhd  <- (lkhd[1,15:lnb,]-min(lkhd[1,15:lnb,]))/(max(lkhd[1,15:lnb,])-min(lkhd[1,15:lnb,]))
    bsamples <- bsamples[15:lnb]
    # assign colorbar label names
    lablocs  <- c(0,1)
    labs     <- c("min","max")
    # location of max likelihood
    loc      <- which(lkhd[1,,]==max(lkhd[1,,]),arr.ind=TRUE)
    # rainblow colors
    numlev   <- 50
    cols     <- rev(rainbow(numlev,start=0,end=1/1.5))    
    # start plotting
    par(mgp=c(2.0,0.5,0))
    par(mar=c(3.5,3.5,2,6))
    filled.contour(bsamples,qsamples,newlkhd, 
                   col=cols,nlevels=numlev,  
                   cex.axis=1.0,plot.axes={axis(1,seq(brange[1],brange[2],by=0.1));
                                           axis(2,seq(qrange[1],qrange[2],by=1.0)); 
                   points(input$xtruepars[1], input$xtruepars[2],pch=24,cex=3,col=rgb(0,0,0),bg=rgb(0.5,0.5,0.5))},
                   plot.title=title(main=titlenm,font.main=1,cex.main=2,
                                    xlab='b parameter [-]',ylab='q parameter [-]', 
                                    cex.lab=2.0),
                   key.title=title('',font.main=1,cex.main=1.0),
                   key.axes=axis(4,at=lablocs,labels=labs,cex.axis=1.5))
    mtext("Relative likelihood [-]", side = 4, cex=1.8, adj=0.3, line=-0.7)
    dev.off()
  }