#' @name paperplot5
#' @aliases paperplot5
#' @title Runs the code for the plots in the publication
#' @description Plots the PDFs of state variables at the point of no return for the flipping case.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot5 <- 
  function(option=1,evalyear=NULL) {
    # initial default values the flipping case
    input          <- setlakedefault(c(1,1),option)
    # define the number of times to repeat an experiment
    numtimes       <- 1
    #estimate the sizes and assign arrays
    nummethods     <- 6
    model_handle   <- input$model_handle
    qtype          <- input$qtype
    seed           <- input$seed
    Bmat           <- input$Bmat
    uinput         <- input$uinput
    numstates      <- input$numstates
    numpars        <- input$numpars
    numbeg         <- numstates+1    
    numvars        <- numpars+numstates                # size of the state variable vector: phosphorus, b, and q, parameters 
    Qmod           <- input$Qmod
    qtype          <- input$qtype
    Qstates        <- getQ(Qmod[1:numstates], numstates)
    plotout        <- array(list(), numtimes*nummethods)
    for (itr in 1:numtimes) {                
      # initiate filter
      input      <- setlakedefault(c(itr,itr),option)
      setup      <- lakesetup(input)
      tipthres   <- 0.5
      # get the data for prediction periods
      if (is.null(evalyear) == TRUE) {
        calyr    <- tippingdetect(input,setup,tipthres)-1   # this is 47 for the lake problem
      } else {
        calyr    <- evalyear  
      }      
      predyr     <- input$tend
      lnpred     <- length(calyr)
      # run the filters
      kalout     <- kalmanfilter(input,setup$z)
      exkalout   <- extendedkalmanfilter(input,setup$z)
      enkalout   <- ensemblekalmanfilter(input,setup$z)
      partout    <- particlefilter(input,setup$z)
      mcmcout    <- runmcmc(logposteriorlake,input,setup,calyr,input$tend,0)             
      # precalibration
      precalout  <- precalibration(input,setup,calyr,predyr)              
      # assign to plot output 
      assigni                <- (itr-1)*nummethods + 1
      plotout[[assigni]]     <- kalout
      plotout[[assigni + 1]] <- exkalout
      plotout[[assigni + 2]] <- enkalout
      plotout[[assigni + 3]] <- partout
      plotout[[assigni + 4]] <- precalout
      plotout[[assigni + 5]] <- mcmcout      
      rm(kalout,exkalout,enkalout,partout,precalout,mcmcout)
      print(itr)
    }
    # save the data
    finaltippredout          <- plotout
    filename                 <- paste('paperplot5_',calyr,'.RData',sep="")
    save(finaltippredout,file=filename)    

    # update numvars to hold pcrit
    numvarsnew <- numvars + 1
    # get plotting data
    # Kalman and Extended Kalman filter
    kaldata    <- array(NA, c(numvarsnew, numtimes, lnpred)) 
    exkaldata  <- array(NA, c(numvarsnew, numtimes, lnpred))     
    kalsd      <- array(NA, c(numvarsnew, numtimes, lnpred))
    exkalsd    <- array(NA, c(numvarsnew, numtimes, lnpred))
    for (itr in 1:numtimes) {      
      tagi                    <- (itr-1)*nummethods+1
      for (period in 1:lnpred) {         
        # now extract full time series starting from the parameter values learnt until time 'calyr'      
        tempval                   <- array(plotout[[tagi]]$x_mean[,1:calyr[period]],c(numvars,calyr[[period]],1))
        tempout                   <- tippingpredict(tempval, input, setup, calyr[period])
        kaldata[1:3,itr,period]   <- tempout                
        # append pcrit at the end
        kaldata[4,itr,period]     <- pcritestimator(tempout[2:3,1])      
        # extended kalman filter
        tempval                   <- array(plotout[[tagi+1]]$x_mean[,1:calyr[period]],c(numvars,calyr[[period]],1))
        tempout                   <- tippingpredict(tempval,input,setup,calyr[period])
        exkaldata[1:3,itr,period] <- tempout
        # append pcrit at the end
        exkaldata[4,itr,period]   <- pcritestimator(tempout[2:3,1])                
        # estimate standard deviations for Kalman and extended Kalman filters
        kalsd[1:3,itr,period]     <- c(sqrt(abs(plotout[[1]]$x_covstate[1,1,calyr[period]])), sqrt(abs(plotout[[1]]$x_covpars[1,1,calyr[period]])),sqrt(abs(plotout[[1]]$x_covpars[2,2,calyr[period]])))    
        exkalsd[1:3,itr,period]   <- c(sqrt(abs(plotout[[2]]$x_covstate[1,1,calyr[period]])), sqrt(abs(plotout[[2]]$x_covpars[1,1,calyr[period]])),sqrt(abs(plotout[[2]]$x_covpars[2,2,calyr[period]])))    
        # in order to estimate the standard deviation of pcrit, generate random realizations (1000) of b and q from given mean and sd and estimate statistics of pcrit
        # for kalman filter
        brand                     <- rnorm(1000, mean=kaldata[2,itr,period])
        qrand                     <- rnorm(1000, mean=kaldata[3,itr,period])
        parvecrand                <- array(c(brand, qrand), c(1000,2))
        pcritrand                 <- pcritestimator(parvecrand) 
        kalsd[4,itr,period]       <- sd(pcritrand)
        rm(brand, qrand, parvecrand, pcritrand)
        # for extended kalman
        brand                     <- rnorm(1000, mean=exkaldata[2,itr,period])
        qrand                     <- rnorm(1000, mean=exkaldata[3,itr,period])
        parvecrand                <- array(c(brand, qrand), c(1000,2))
        pcritrand                 <- pcritestimator(parvecrand) 
        exkalsd[4,itr,period]     <- sd(pcritrand)
        rm(brand, qrand, parvecrand, pcritrand)
        # for ensemble Kalman filter        
        numens                    <- dim(plotout[[tagi+2]]$x_enkfall[[calyr[period]]])[1]
        tempval                   <- array(NA,c(numvars,predyr,numens))
        for (tempi in 1:predyr) {
          tempval[,tempi,]        <- t(plotout[[tagi+2]]$x_enkfall[[calyr[period]]])
        }
        tempout                   <- tippingpredict(tempval,input,setup,calyr[period])
        enkaldata                 <- array(NA,c(numvarsnew,numtimes,lnpred,numens))
        enkaldata[1:3,itr,period,]<- tempout
        temp                      <- array(t(tempout[2:3,]),c(dim(tempval)[3],2))
        enkaldata[4,itr,period,]  <- pcritestimator(temp)
        rm(tempval,tempout,temp)
        # for particle filter        
        numpar                    <- dim(plotout[[tagi+3]]$x_parall[[calyr[period]]])[1]
        tempval                   <- array(NA,c(numvars,predyr,numpar))
        for (tempi in 1:predyr) {
          tempval[,tempi,]        <- t(plotout[[tagi+3]]$x_parall[[calyr[period]]])
        }
        tempout                   <- tippingpredict(tempval,input,setup,calyr[period])
        partdata                  <- array(NA,c(numvarsnew,numtimes,lnpred,numpar))
        partdata[1:3,itr,period,] <- tempout        
        temp                      <- array(t(tempout[2:3,]),c(dim(tempval)[3],2))
        partdata[4,itr,period,]   <- pcritestimator(temp)   
        print(period)
        rm(tempval, tempout, temp)
      }
      print(itr)
    }    
    # save full workspace
    filename      <- paste('paperplot5_',calyr,'all.RData',sep="")
    save.image(filename)
    
    # now plot and save as pdf
    filename      <- paste('Figure5_',calyr,'.pdf',sep="")
    cairo_pdf(filename, width=6, height=6)
    # labels for parameters
    labelpar   <- c('Phosphorus [-]','b parameter [-]','q parameter [-]','Critical threshold [-]')
    #colors and labels for the methods
    labelm     <- c('Kalman','ExtendedK','EnsembleK','Particle','Precalib','MCMC')    
    colsm      <-  c(rgb(1,0.5,0),rgb(1,0,0),rgb(0,0.8,0),rgb(0,1,1),rgb(0.7,0.7,0.7),rgb(0,0,1))
    #labels and linetypes for the periods
    label      <- calyr
    ltyp       <- c(1)    
    lwdval     <- 2
    # plot settings and empty base plot    
    par(mfrow = c(2,2))
    par(mar = c(4,4.5,2,1)+0.1)
    par(mgp = c(2.5,1,0))
    # add the plots
    # Kalman and Extended Kalman filter
    for (itr in 1:numtimes) {    
      tagi       <- (itr-1)*nummethods+1
      for (parnum in 1:numvarsnew) {    
        print(parnum)
        if (parnum==1) {
          tempx  <- seq(-0.1,10.0,length=100)
          tempy  <- seq(0,100,length=100)
          ymax   <- 1.5
        } else if (parnum==2) {
          tempx  <- seq(0.0,0.8,length=100)
          tempy  <- seq(0,100,length=100)        
          ymax   <- 20
        } else if (parnum==3) {
          tempx  <- seq(0.0,8.0,length=100)
          tempy  <- seq(0,100,length=100)        
          ymax   <- 2.0
        } else {
          tempx  <- seq(0.0,1.5,length=100)
          tempy  <- seq(0,100,length=100)        
          ymax   <- 15
        }
        plot(tempx,tempy,col='white',xlab=labelpar[parnum],ylab='Density [-]',
             cex.lab=1.5,cex.axis=1.5,xlim=c(0,max(tempx)),ylim=c(0,ymax),yaxp=c(0,ymax,1))
        for (period in 1:lnpred) {
          # kalman filter
          #           kaly         <- dnorm(tempx, mean=kaldata[parnum,itr,period],   sd=kalsd[parnum,itr,period])
          #           lines(tempx, kaly,   lwd=lwdval, col = colsm[1], lty=ltyp[period])
          # extended kalman filter
          #           exkaly       <- dnorm(tempx, mean=exkaldata[parnum,itr,period], sd=exkalsd[parnum,itr,period])
          #           lines(tempx, exkaly, lwd=lwdval, col = colsm[2], lty=ltyp[period])
          # ensemble Kalman filter
          endval       <- density(enkaldata[parnum,itr,period,],na.rm=TRUE,bw=sd(enkaldata[parnum,itr,period,],na.rm=TRUE)/2)
          lines(endval$x, endval$y, lwd=lwdval, col=colsm[3], lty=ltyp[period])
          # particle filer
          partdval     <- density(partdata[parnum,itr,period,],na.rm=TRUE,bw=sd(partdata[parnum,itr,period,],na.rm=TRUE)/2)
          lines(partdval$x,partdval$y,lwd=lwdval,col=colsm[4],lty=ltyp[period])             
          # precalibration
          if (parnum==4) {
            tempval       <- t(plotout[[tagi+4]]$xpar[[calyr]][2:3,])
            precalibdata  <- pcritestimator(tempval)            
          } else {
            precalibdata  <- plotout[[tagi+4]]$xpar[[calyr]][parnum,]            
          }         
          precval          <- density(precalibdata,na.rm=TRUE,bw=sd(precalibdata,na.rm=TRUE)/2)
          lines(precval$x,precval$y,lwd=lwdval,col=colsm[5],lty=ltyp[period])          
          # mcmc
          if (parnum==4) {
            tempval       <- t(plotout[[tagi+5]]$outlast[2:3,calyr,])
            mcmcdata      <- pcritestimator(tempval)
          } else {
            mcmcdata      <- plotout[[tagi+5]]$outlast[parnum,predyr,]         
          }                   
          mcmcval      <- density(mcmcdata,na.rm=TRUE,bw=sd(mcmcdata,na.rm=TRUE)/2)
          lines(mcmcval$x,mcmcval$y,lwd=lwdval,col=colsm[6],lty=ltyp[period])          
          # truth
          if (parnum<4) {
            lines(c(setup$xtrue[parnum,predyr[period]],setup$xtrue[parnum, predyr[period]]),c(0,100),lwd=lwdval,lty=3,col=rgb(0,0,0))
          } else {
            truethres <- pcritestimator(setup$xtrue[2:3,predyr[period]])
            lines(c(truethres,truethres),c(0,100),lwd=lwdval,lty=3, col = rgb(0,0,0))  
          }
          rm(endval, partdval,precval,mcmcval,precalibdata,mcmcdata)
        }
        if (parnum==4) {
          legend(0.67,15,c("EnsembleK","Particle","Precalib","MCMC","Truth"),box.col=rgb(1,1,1),
                 bg=rgb(1,1,1,0),y.intersp=1,x.intersp=0.7,lwd=c(2,2,2,2,2),lty=c(1,1,1,1,3),
                 pch=c(26,26,26,26,26),col=c(colsm[3:6],rgb(0,0,0)),cex=1.0,ncol=1)
        }
        rm(tempx, tempy)
      }    
    }
    dev.off()    
  }