#' @name paperplot6par
#' @aliases paperplot6par
#' @title Runs the code for Figure 6 - parallel version for execution
#' @description Plotting the predicting probability of eutrophication for each method as a function of learning time.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot6par <- 
  function(option=1) {
    input          <- setlakedefault(c(1,1),option)
    # define the number of times to repeat an experiment
    numtimes       <- input$numtimes
    #estimate the sizes and assign arrays
    nummethods     <- 5
    logposteriormod<- input$logposteriormod    
    model_handle   <- input$model_handle
    numstates      <- input$numstates
    numpars        <- input$numpars
    numvars        <- numpars+numstates
    qtype          <- input$qtype
    seed           <- input$seed
    Bmat           <- input$Bmat
    uinput         <- input$uinput
    numbeg         <- numstates+1    
    numvars        <- numpars+numstates       
    Qmod           <- input$Qmod
    qtype          <- input$qtype
    Qstates        <- getQ(Qmod[1:numstates], numstates)
    N              <- length(seq(0,input$tend))
    tipthres       <- 0.5
    # set the years until which learning is allowed
    calyr          <- c(1,10,20,30,40,50,60,70,80,90)
    tipyear        <- tippingdetect(input,setup,tipthres)  # assess the point of no return
    predyr         <- input$tend
    lnpred         <- length(calyr)      
    ###################### parallelization stuff ######################
    no_cores       <- detectCores() - 1
    cl             <- makeCluster(numtimes,outfile="debug.txt")
    registerDoParallel(cl)
    ###################### run the 6 methods ###########################
    enkalout <- foreach(itr=1:numtimes,.combine='rbind',.packages='learningmodule') %dopar% {                
      # initiate filter
      input      <- setlakedefault(c(itr,itr),option)
      setup      <- lakesetup(input)
      # run the filters
      ensemblekalmanfilter(input,setup$z)
    }
    partout <- foreach(itr=1:numtimes,.combine='rbind',.packages='learningmodule') %dopar% {                
      # initiate filter
      input      <- setlakedefault(c(itr,itr),option)
      setup      <- lakesetup(input)
      # run the filters
      particlefilter(input,setup$z)
    }    
    mcmcout <- foreach(itr=1:numtimes,.combine='rbind',.packages='learningmodule') %dopar% {                
      # initiate filter
      input      <- setlakedefault(c(itr,itr),option)
      setup      <- lakesetup(input)
      # run the filters
      runmcmc(logposteriormod,input,setup,predyr,predyr,option=0)
    }
    # 2nd mcmc with more samples
    mcmcout2 <- foreach(itr=1:numtimes,.combine='rbind',.packages='learningmodule') %dopar% {                
      # initiate filter
      input          <- setlakedefault(c(itr,itr),option)
      setup          <- lakesetup(input)
      input2         <- input
      input2$nummcmc <- 25000
      # run the filters
      runmcmc(logposteriormod,input2,setup,predyr,predyr,option=0)
    }  
    # end parallelization 
    stopImplicitCluster()
    # reset for precalib
    no_cores       <- detectCores() - 1
    cl             <- makeCluster(numtimes,outfile="debug.txt")
    registerDoParallel(cl)
    precalout   <- foreach(itr=1:numtimes,.combine='rbind',.packages='learningmodule') %dopar% {
      # initiate filter
      input      <- setlakedefault(c(itr,itr),option)
      setup      <- lakesetup(input)
      # run the precalibration
      precalibration(input,setup,predyr,predyr)  
    }      
    # end parallelization 
    stopImplicitCluster()    
    # assign to plot output 
    plotout        <- array(list(),numtimes*nummethods)   
    for (itr in 1:numtimes) {
      assigni              <- (itr-1)*nummethods + 1
      plotout[[assigni]]   <- enkalout[itr,]
      plotout[[assigni+1]] <- partout[itr,]
      plotout[[assigni+2]] <- precalout[itr,]
      plotout[[assigni+3]] <- mcmcout[itr,]
      plotout[[assigni+4]] <- mcmcout2[itr,]
    }  
    # now identify the MCMC predicted state and parameters for learning until time period specified by 'calyr'    
    chainout       <- array(list(),dim=c(numtimes,lnpred))      
    chainout2      <- array(list(),dim=c(numtimes,lnpred))      
    timerange      <- seq(input$tbeg,input$tend,by=input$timestep)        # define the timerange to run the model
    N              <- length(timerange)                                   # total number of time steps            
    for (itr in 1:numtimes) {                
      input        <- setlakedefault(c(itr,itr),option)   
      setup        <- lakesetup(input)
      # initiate filterfor (period in 1:lnpred) {      
      assigni      <- (itr-1)*nummethods + 1
      for (period in 1:lnpred) { 
        numchain1      <- dim(plotout[[assigni+3]]$outall[[calyr[period]]])[2]          # total number of chain (last 50%) 
        lastval        <- array(NA,c(numvars,numchain1))
        for (j in 1:numchain1) {
          temp         <- array(NA,dim=c(numvars,N))    
          temp[,1]     <- plotout[[assigni+3]]$outall[[calyr[period]]][,j]
          for (i in input$tbeg:input$tend) {  
            if (i==1) {
              temp[1:numstates,i]      <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+i,temp[1:numstates,i],temp[numbeg:numvars,i],timestep,i)+Bmat*uinput[i]          
            } else {
              temp[1:numstates,i]      <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+i,temp[1:numstates,i-1],temp[numbeg:numvars,i-1],timestep,i)+Bmat*uinput[i]        
              temp[numbeg:numvars,i]   <- temp[numbeg:numvars,i-1]
            }
          }
          lastval[,j]                  <- temp[,N]
          rm(temp,temp2)        
        }
        chainout[[itr,period]]       <- lastval
      }              
      for (period in 1:lnpred) { 
        numchain2      <- dim(plotout[[assigni+4]]$outall[[calyr[period]]])[2]          # total number of chain (last 50%) 
        lastval2       <- array(NA,c(numvars,numchain2))
        for (j in 1:numchain2) {
          temp2        <- array(NA,dim=c(numvars,N))    
          temp2[,1]    <- plotout[[assigni+4]]$outall[[calyr[period]]][,j]
          for (i in input$tbeg:input$tend) {  
            if (i==1) {
              temp2[1:numstates,i]     <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+i,temp2[1:numstates,i],temp2[numbeg:numvars,i],timestep,i)+Bmat*uinput[i]          
            } else {
              temp2[1:numstates,i]     <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+i,temp2[1:numstates,i-1],temp2[numbeg:numvars,i-1],timestep,i)+Bmat*uinput[i]        
              temp2[numbeg:numvars,i]  <- temp2[numbeg:numvars,i-1]
            }
          }
          lastval2[,j]                 <- temp2[,N]
          rm(temp2)        
        }
        chainout2[[itr,period]]      <- lastval2
      }        
    }
    plotout$chainout     <- chainout
    plotout$chainout2    <- chainout2 
    
    # save plotout
    save(plotout,file=paste('paperplot6_plt',numtimes,'.Rdata',sep=""))
    # save all files, just a backup
    save(list=ls(all.names=TRUE),file=paste('paperplot6_',numtimes,'.Rdata',sep=""))
    ###############################################################################
    
    ########### extract the data for plotting  from method output #################    
    enkaldata  <- array(list(),c(numtimes,lnpred))
    partdata   <- array(list(),c(numtimes,lnpred))
    for (itr in 1:numtimes) {      
      input          <- setlakedefault(c(itr,itr),option)   
      setup          <- lakesetup(input)
      tagi           <- (itr-1)*nummethods+1
      for (period in 1:lnpred) {         
        # now extract full time series starting from the parameter values learnt until time 'calyr'      
        # for ensemble Kalman       
        numens                    <- dim(plotout[[tagi]]$x_enkfall[[calyr[period]]])[1]
        tempval                   <- array(NA,c(numvars,predyr,numens))
        for (tempi in 1:predyr) {
          tempval[,tempi,]        <- t(plotout[[tagi]]$x_enkfall[[calyr[period]]])
        }
        tempout                   <- tippingpredict(tempval,input,setup,calyr[period])        
        enkaldata[[itr,period]]   <- tempout
        rm(tempval)
        # particle filter
        numpar                    <- dim(plotout[[tagi+1]]$x_parall[[calyr[period]]])[1]
        tempval                   <- array(NA,c(numvars,predyr,numpar))
        for (tempi in 1:predyr) {
          tempval[,tempi,]        <- t(plotout[[tagi+1]]$x_parall[[calyr[period]]])
        }
        tempout                   <- tippingpredict(tempval,input,setup,calyr[period])
        partdata[[itr,period]]    <- tempout        
        rm(tempval)
        print(paste("period ",period))
      }
      print(paste("itr ", itr))
    }
    #############################################################################
    ########################  get the prior probabilities #######################
    # priors at t=0    
    # prior probability of eutrophication for Kalman and Extended Kalman
    # Generate random samples from the initial state, propagate in future, estimate probabilities
    # the number of random samples generated is the same as the number of ensemble members for the ensemble kalman filter
    initprior       <- rep(NA,numtimes)
    enkprior        <- rep(NA,numtimes)
    pfprior         <- rep(NA,numtimes)
    precprior       <- rep(NA,numtimes)
    mcmcprior       <- rep(NA,numtimes)
    mcmc2prior      <- rep(NA,numtimes) 
    Pguess          <- diag(c(input$Pxguess,diag(input$Pxguesspars)))
    xguess          <- c(input$xguess,input$xguesspars)
    for (itr in 1:numtimes) {
      input          <- setlakedefault(c(itr,itr),option)   
      setup          <- lakesetup(input)
      randdata       <- multirandnorm(xguess,Pguess,input$n_ens,1)
      tempinp        <- rep(NA,input$n_ens)
      for (tempi in 1:input$n_ens) {
        tempval              <- array(randdata[tempi,],c(numvars,1,1))
        temp                 <- tippingpredict(tempval,input,setup,0)
        tempinp[tempi]       <- temp[1]
        rm(tempval,temp)
      }
      initprior[itr]  <- probeutro(tempinp,input,setup,tipthres,input$n_ens)
      rm(tempinp,randdata)
      # next, get the prior probabilities for the ensemble Kalman and the particle filters
      # these can be different from the KF/EKF priors as the EnKF/PF can deal with lognormal distributions
      if (input$prior == 0) {
        enkprior[itr] <- initprior[itr]
      } else {
        location     <- rep(NA,numvars)
        scale        <- rep(NA,numvars)
        for (i in 1:numvars) {
          location[i]     <- log(xguess[i]/sqrt(1+Pguess[i,i]/xguess[i]^2))
          scale[i]        <- sqrt(log(1+Pguess[i,i]/xguess[i]^2))
        }
        scale        <- diag(scale)
        randdata2    <- multirandnorm(location,scale,input$n_ens,1)
        randdata2    <- exp(randdata2)
        tempinp2     <- rep(NA,input$n_ens)
        tempval      <- array(t(randdata2),c(numvars,1,input$n_ens))
        temp         <- tippingpredict(tempval,input,setup,0)
        tempinp2     <- temp[1,]
        rm(tempval,temp)    
        enkprior[itr]<- probeutro(tempinp2,input,setup,tipthres,input$n_ens)
        rm(tempinp2,location,scale,randdata2)  
      }
      pfprior[itr]       <- enkprior[itr]
      # prior for precabliration is also the same
      precprior[itr]     <- enkprior[itr]
      # prior for mcmc is the same as that of ensemble KF 
      mcmcprior[itr]     <- enkprior[itr]
      mcmc2prior[itr]    <- enkprior[itr]
      print(itr)
    }
    # initialize the prior vector
    priorvec      <- c(enkprior[1],pfprior[1],precprior[1],mcmcprior[1],mcmc2prior[1])    
    ##############################################################################    
    # save all files, just a backup
    save(list=ls(all.names=TRUE),file=paste('paperplot6_',numtimes,'all.Rdata',sep=""))
    ######################## make the plot #######################################
    # save as pdf
    cairo_pdf(paste("Figure6.pdf", sep=""), width=10, height=6)
    #colors and labels for the methods
    labelm     <- c('EnsembleK','Particle','Precalib','MCMC','MCMC2')    
    colsm      <- c(rgb(0,0.8,0),rgb(0,1,1),rgb(0.5,0.5,0.5),rgb(0,0,1),rgb(0,0.5,1))    
    colsm2     <- c(rgb(0,0.8,0,0.5),rgb(0,1,1,0.5),rgb(0.5,0.5,0.5,0.5),rgb(0,0,1,0.5),rgb(0,0.5,1,0.5))    
    pchv       <- c(1,1,1,1,4)
    sizevec    <- c(2,2,2,2,2)
    #labels and linetypes for the periods
    label      <- calyr
    ltyp       <- 1  
    lwdval     <- 2   
    numtoplot  <- 5
    notipyear  <- tipyear-1
    # plot settings and empty base plot        
    par(mar = c(4,4.5,2,3)+0.1)
    par(mgp = c(2.5,1,0))
    # x location
    xlocs       <- calyr
    ylocs       <- xlocs
    jitterx     <- c(-2.0,-1.0,0,1.0,2.0)
    ymax        <- 1.1
    par         <- 1
    # initiate vectors for saving outputs to draw lines
    endval      <- array(NA,c(lnpred,numtimes))   
    partdval    <- array(NA,c(lnpred,numtimes))
    precval     <- array(NA,c(lnpred,numtimes))
    mcmcval     <- array(NA,c(lnpred,numtimes))
    mcmcval2    <- array(NA,c(lnpred,numtimes))
    # assign first value to prior
    endval[1,]  <- priorvec[1]
    partdval[1,]<- priorvec[2]
    precval[1,] <- priorvec[3]
    mcmcval[1,] <- priorvec[4]  
    mcmcval2[1,]<- priorvec[5]
    #make the base plot
    plot(xlocs,ylocs,ylim=c(0,ymax+0.0),col=rgb(1,1,1,0),xlab='Learning time [years]',ylab='Probability of eutrophication [-]',
         cex.lab=1.5, cex.axis=1.5, axes='FALSE')
    # plot the prior vectors
    points(rep(xlocs[1],numtoplot),priorvec,lwd=lwdval,col=colsm,bg=colsm,pch=pchv,cex=sizevec)
    for (itr in 1:numtimes) {    
      tagi                   <- (itr-1)*nummethods + 1
      # estimate and plot the probabilities
      for (period in 2:lnpred) {
        # kalman filter
        # kaly[period,itr]     <- pnorm(tipthres, mean=kaldata[par,itr,period],   sd=kalsd[par,itr,period])
        # extended kalman filter
        # exkaly[period,itr]   <- pnorm(tipthres, mean=exkaldata[par,itr,period], sd=exkalsd[par,itr,period])
        # ensemble Kalman 
        endval[period,itr]   <- probeutro(enkaldata[[itr,period]][1,],input,setup,tipthres,length(enkaldata[[itr,period]][1,]))
        # particle filter
        partdval[period,itr] <- probeutro(partdata[[itr,period]][1,],input,setup,tipthres,length(partdata[[itr,period]][1,]))
        # precalibration
        precalibdata         <- plotout[[tagi+2]]$xpar[[calyr[period]]][par,]
        precval[period,itr]  <- probeutro(precalibdata,input,setup,tipthres,length(precalibdata))
        # mcmc
        mcmcdata             <- chainout[[itr,period]][par,]
        mcmcval[period,itr]  <- probeutro(mcmcdata,input,setup,tipthres,length(mcmcdata))
        if (mcmcval[period,itr]==Inf) {mcmcval[period,itr] = ymax}
        rm(precalibdata,mcmcdata)
        # mcmc2
        mcmcdata2            <- chainout2[[itr,period]][par,]
        mcmcval2[period,itr] <- probeutro(mcmcdata2,input,setup,tipthres,length(mcmcdata2))
        if (mcmcval2[period,itr]==Inf) {mcmcval2[period,itr] = ymax}
        rm(mcmcdata2)
      }
    }
    for (period in 2:lnpred) {
      # points(xlocs[period], mean(kaly[period,]), col = colsm[1], bg=colsm[1], pch=pchv[1], cex=2)
      # points(xlocs[period], mean(exkaly[period,]), col = colsm[2], bg=colsm[2], pch=pchv[2], cex=2)
      points(xlocs[period]+jitterx[1], mean(endval[period,]),  col=colsm[1],bg=colsm[1],pch=pchv[1],cex=2)
      points(xlocs[period]+jitterx[2], mean(partdval[period,]),col=colsm[2],bg=colsm[2],pch=pchv[2],cex=2)
      points(xlocs[period]+jitterx[3], mean(precval[period,]), col=colsm[3],bg=colsm[3],pch=pchv[3],cex=2)
      points(xlocs[period]+jitterx[4], mean(mcmcval[period,]), col=colsm[4],bg=colsm[4],pch=pchv[4],cex=2)
      points(xlocs[period]+jitterx[5], mean(mcmcval2[period,]),col=colsm[5],bg=colsm[5],pch=pchv[5],cex=2,lwd=2)
      # adding in the lines
      # lines(c(xlocs[period],xlocs[period]), range(kaly[period,]),     lwd=1, col=colsm2[1])
      # lines(c(xlocs[period],xlocs[period]), range(exkaly[period,]),   lwd=1, col=colsm2[2])
      lines(c(xlocs[period],xlocs[period])+jitterx[1], range(endval[period,]),  lwd=2,col=colsm2[1])
      lines(c(xlocs[period],xlocs[period])+jitterx[2], range(partdval[period,]),lwd=2,col=colsm2[2])
      lines(c(xlocs[period],xlocs[period])+jitterx[3], range(precval[period,]), lwd=2,col=colsm2[3])
      lines(c(xlocs[period],xlocs[period])+jitterx[4], range(mcmcval[period,]), lwd=2,col=colsm2[4])
      lines(c(xlocs[period],xlocs[period])+jitterx[5], range(mcmcval2[period,]),lwd=2,col=colsm2[5])
    }
    # add in the horizontal line at the point of no return
    lines(c(notipyear,notipyear),c(-0.5,ymax-0.05),lwd=lwdval,lty=2,col=rgb(0.8,0,0))
    # add in the horizontal line at probability = 0.5
    # lines(calyr, rep(0.5, length(calyr)), lwd = lwdval, lty=2, col = rgb(0.5,0.5,0.5))
    # line at probability of eutrophication = 1, this is the truth
    lines(calyr,rep(1,length(calyr)),lwd=lwdval,lty=2,col=rgb(0,0,0))
    # add in the legend and guiding text on the plot
    legend(calyr[6],0.4,labelm,box.col=rgb(0,0,0),bg=NA,y.intersp=1,x.intersp=0.7,
           pch=pchv,col=colsm,pt.bg=colsm,pt.cex=1.5,ncol=2,cex=1.2)
    arrows(notipyear+6,ymax-1.00,x1=tipyear,y1=ymax-1.00,length=0.15,angle=30,code=2,
           col=rgb(0.8,0,0),lty=1,lwd=2)
    text(notipyear+16,ymax-0.99,'Time of no return',col=rgb(0.8,0,0),cex=1.2)
    text(notipyear-30, 1.04,'True state',cex=1.2)   
    axis(side=1,at=c(0,calyr[2:length(calyr)]))
    axis(side=2,at=c(0,0.5,1))
    dev.off()   
    ##############################################################################
  }