#' @name paperplot_Sup1
#' @aliases paperplot_Sup1
#' @title Runs the code for the supplementary figures in the publication.
#' @description Runs the code for plotting the mean and 90% CI for each method.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot_Sup1 <- 
  function(parplot=1) {
    # choose the parameter to plot
    figcount       <- 1+(parplot-1)*3
    ylabs          <- c('Phosphorus [units]','b parameter [units]','q parameter [units]')
    z_90_normal    <- qnorm(0.95)
    numplt         <- 2
    filterlimcol   <- rgb(0,0,0)
    filtermeancol  <- rgb(0,0,0)
    truthcol       <- rgb(0,0,1)
    obscol         <- rgb(0,0,1,0.5)
    # ensemble kalman filter
    cairo_pdf(paste("FigureA",figcount,"_",parplot,".pdf", sep=""), width=3, height=3)
    mat            <- matrix(seq(1,numplt,length=numplt),numplt,1,byrow=TRUE)
    layout(mat,widths=c(rep(1,numplt)),heights=c(rep(1, numplt)))
    option         <- 0
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2a.RData')
    ############################ Ensemble Kalman filter ####################
    # plot analysis mean and 90% CI for the ensemble Kalman filter
    uplimits       <- filterout$ensemblekalmanout$x_mean[parplot,]+sqrt(filterout$ensemblekalmanout$x_cov[parplot,parplot,])*z_90_normal
    lwlimits       <- filterout$ensemblekalmanout$x_mean[parplot,]-sqrt(filterout$ensemblekalmanout$x_cov[parplot,parplot,])*z_90_normal
    if (parplot==1) {
      ylimvals       <- range(setup$z)
      ylimvals[2]    <- 0.5
    } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean    
    plot(filterout$ensemblekalmanout$x_mean[parplot,],ylim=ylimvals,cex.main=0.8,frame.plot=0,
         pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]',col=filtermeancol)
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    # insert text
    mtext('a) Low emissions',side=3,line=-1.2,at=80)
    mtext('Ensemble Kalman filter',side=3,line=-0.7,at=45)        
    # case b
    option         <- 1
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2b.RData')
    # plot analysis mean and 90% CI for the ensemble Kalman filter
    uplimits       <- filterout$ensemblekalmanout$x_mean[parplot,]+sqrt(filterout$ensemblekalmanout$x_cov[parplot,parplot,])*z_90_normal
    lwlimits       <- filterout$ensemblekalmanout$x_mean[parplot,]-sqrt(filterout$ensemblekalmanout$x_cov[parplot,parplot,])*z_90_normal
    if (parplot==1) {
      ylimvals       <- range(setup$z)
      ylimvals[2]    <- 3.0
    } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean
    plot(filterout$ensemblekalmanout$x_mean[parplot,],ylim=ylimvals,frame.plot=0,
         pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]',col=filtermeancol)
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    # insert legend
    if (parplot==1) {
      legend(60,ylimvals[2]/2,c('Analysis mean','90% CI','Truth','Observations'),
           lty=c(0,1,1,0),col=c(filtermeancol,filterlimcol,truthcol,obscol),
           pch=c(4,26,26,1),cex=0.7,ncol=1,bty="n",y.intersp=0.7)
    } else if (parplot==2) {
      legend(65,ylimvals[2]/2.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)       
    } else if (parplot==3) {
      legend(65,ylimvals[2]/1.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)       
    }
    mtext('b) High emissions',side=3,line=-1.2,at=80)
    dev.off()
    
    # particle filter
    figcount       <- 2+(parplot-1)*3
    cairo_pdf(paste("FigureA",figcount,"_",parplot,".pdf", sep=""), width=3, height=3)
    mat            <- matrix(seq(1,numplt,length=numplt),numplt,1,byrow=TRUE)
    layout(mat,widths=c(rep(1,numplt)),heights=c(rep(1, numplt)))
    option         <- 0
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2a.RData')
    # estimate limits
    uplimits       <- filterout$particleout$x_mean[parplot,]+sqrt(filterout$particleout$x_cov[parplot,parplot,])*z_90_normal
    lwlimits       <- filterout$particleout$x_mean[parplot,]-sqrt(filterout$particleout$x_cov[parplot,parplot,])*z_90_normal
    if (parplot==1) {
      ylimvals       <- range(c(setup$z,uplimits,lwlimits))
      ylimvals[2]    <- 0.5
    } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean    
    plot(filterout$particleout$x_mean[parplot,],ylim=ylimvals,col=filtermeancol,
         frame.plot=0,pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]')
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    # insert text
    mtext('a) Low emissions',side=3,line=-1.2,at=80)
    mtext('Particle filter',side=3,line=-0.7,at=45)            
    # case b
    option         <- 1
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2b.RData')
    # estimate the sd
    uplimits       <- filterout$particleout$x_mean[parplot,]+sqrt(filterout$particleout$x_cov[parplot,parplot,])*z_90_normal
    lwlimits       <- filterout$particleout$x_mean[parplot,]-sqrt(filterout$particleout$x_cov[parplot,parplot,])*z_90_normal    
    if (parplot==1) {
      ylimvals       <- range(c(setup$z,uplimits,lwlimits))
      ylimvals[2]    <- 3.0
      } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }   
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean    
    plot(filterout$particleout$x_mean[parplot,],ylim=ylimvals,col=filtermeancol,
         frame.plot=0,pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]')
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    # insert legend
    if (parplot==1) {
      legend(60,ylimvals[2]/2,c('Analysis mean','90% CI','Truth','Observations'),
             lty=c(0,1,1,0),col=c(filtermeancol,filterlimcol,truthcol,obscol),
             pch=c(4,26,26,1),cex=0.7,ncol=1,bty="n",y.intersp=0.7)
    } else if (parplot==2) {
      legend(65,ylimvals[2]/2.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)       
    } else if (parplot==3) {
      legend(65,ylimvals[2]/1.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)       
    }   
    mtext('b) High emissions',side=3,line=-1.2,at=80)
    dev.off()  
    #########################################################################
    
    # Precalibration
    figcount       <- 3+(parplot-1)*3
    cairo_pdf(paste("FigureA",figcount,"_",parplot,".pdf", sep=""), width=3, height=3)
    mat            <- matrix(seq(1,numplt,length=numplt),numplt,1,byrow=TRUE)
    layout(mat,widths=c(rep(1,numplt)),heights=c(rep(1, numplt)))
    option         <- 0
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2a.RData')
    # estimate upper and lower limits    
    uplimits       <- filterout$precalib$x_mean[parplot,]+sqrt(filterout$precalib$x_cov[parplot,])*z_90_normal
    lwlimits       <- filterout$precalib$x_mean[parplot,]-sqrt(filterout$precalib$x_cov[parplot,])*z_90_normal
    if (parplot==1) {
      ylimvals       <- range(c(setup$z,uplimits,lwlimits))
      ylimvals[2]    <- 0.5      
    } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean    
    plot(filterout$precalib$x_mean[parplot,],ylim=ylimvals,frame.plot=0,
         pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]')
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    # insert text
    mtext('a) Low emissions',side=3,line=-1.2,at=80)
    mtext('Precalibration',side=3,line=-0.7,at=45)            
    # case b
    option         <- 1
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2b.RData')
    # estimate upper and lower limits    
    uplimits       <- filterout$precalib$x_mean[parplot,]+sqrt(filterout$precalib$x_cov[parplot,])*z_90_normal
    lwlimits       <- filterout$precalib$x_mean[parplot,]-sqrt(filterout$precalib$x_cov[parplot,])*z_90_normal
    if (parplot==1) {
      ylimvals       <- range(c(setup$z,uplimits,lwlimits))          
    } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean    
    plot(filterout$precalib$x_mean[parplot,],ylim=ylimvals,frame.plot=0,col=filtermeancol,
         pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]')
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    # insert legend
    if (parplot==1) {
      legend(60,ylimvals[2]/2,c('Analysis mean','90% CI','Truth','Observations'),
           lty=c(0,1,1,0),col=c(filtermeancol,filterlimcol,truthcol,obscol),
           pch=c(4,26,26,1),cex=0.7,ncol=1,bty="n",y.intersp=0.7)
    } else if (parplot==2) {
      legend(65,ylimvals[2]/2.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)    
    } else if (parplot==3) {
      legend(65,ylimvals[2]/1.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)       
    }
    mtext('b) High emissions',side=3,line=-1.2,at=80)
    dev.off()
    
    # MCMC 
    figcount       <- 4+(parplot-1)*3
    cairo_pdf(paste("FigureA",figcount,"_",parplot,".pdf", sep=""), width=3, height=3)
    mat            <- matrix(seq(1,numplt,length=numplt),numplt,1,byrow=TRUE)
    layout(mat,widths=c(rep(1,numplt)),heights=c(rep(1, numplt)))
    option         <- 0
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2a.RData')
    uplimits       <- filterout$mcmcout$x_mean[parplot,]+sqrt(filterout$mcmcout$x_cov[parplot,])*z_90_normal
    lwlimits       <- filterout$mcmcout$x_mean[parplot,]-sqrt(filterout$mcmcout$x_cov[parplot,])*z_90_normal    
    if (parplot==1) {
      ylimvals       <- range(c(setup$z,uplimits,lwlimits))    
    } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean    
    plot(filterout$mcmcout$x_mean[parplot,],ylim=ylimvals,frame.plot=0,col=filtermeancol,
         pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]')
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    # insert text
    mtext('a) Low emissions',side=3,line=-1.2,at=80)
    mtext('MCMC',side=3,line=-0.7,at=45)            
    #########################################################################
    option         <- 1
    input          <- setlakedefault(c(1,1),option)
    setup          <- lakesetup(input)
    load('paperplot2b.RData')
    uplimits       <- filterout$mcmcout$x_mean[parplot,]+sqrt(filterout$mcmcout$x_cov[parplot,])*z_90_normal
    lwlimits       <- filterout$mcmcout$x_mean[parplot,]-sqrt(filterout$mcmcout$x_cov[parplot,])*z_90_normal    
    if (parplot==1) {
      ylimvals       <- range(setup$z)
      ylimvals[2]    <- 3.0      
    } else if (parplot==2) {
      ylimvals       <- c(0.0,1.0)      
    } else if (parplot==3) {
      ylimvals       <- c(0.0,10.0)
    }
    # start plot
    par(family="sans",mar=c(2.0,2.0,0.1,0.1),mgp=c(1.0,0.4,0),ps=9)
    # plot the mean    
    plot(filterout$mcmcout$x_mean[parplot,],ylim=ylimvals,col=filtermeancol,frame.plot=0,
         pch=4,cex=0.7,ylab=ylabs[parplot],xlab='Time [years]')
    # plot the upper and lower limits
    lines(uplimits,col=filterlimcol)
    lines(lwlimits,col=filterlimcol)
    # plot the truth
    lines(setup$xtrue[parplot,],col=truthcol)
    # plot the observations
    if (parplot==1) {
      points(setup$z,col=obscol,cex=0.7)
    }
    if (parplot==1) {
      legend(60,ylimvals[2]/2,c('Analysis mean','90% CI','Truth','Observations'),
             lty=c(0,1,1,0),col=c(filtermeancol,filterlimcol,truthcol,obscol),
             pch=c(4,26,26,1),cex=0.7,ncol=1,bty="n",y.intersp=0.7)
    } else if (parplot==2) {
      legend(65,ylimvals[2]/2.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)    
    }  else if (parplot==3) {
      legend(65,ylimvals[2]/1.5,c('Analysis mean','90% CI','Truth'),
             lty=c(0,1,1),col=c(filtermeancol,filterlimcol,truthcol),
             pch=c(4,26,26),cex=0.7,ncol=1,bty="n",y.intersp=0.7)       
    }
    mtext('b) High emissions',side=3,line=-1.2,at=80)
    dev.off()
  }