#' @name paperplot_Sup2
#' @aliases paperplot_Sup2
#' @title Runs the code for the supplementary figures in the publication.
#' @description This function runs the code for generating the random realizations used in the analysis for Figures 4 and 6.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot_Sup2 <- 
  function(option=1) {
    # generate the policies     
    input           <- setlakedefault(c(1,1),option)
    # define the number of times to repeat an experiment
    numtimes        <- 10
    #estimate the sizes and assign arrays    
    plotout         <- array(NA, c(numtimes,input$tend))
    ###################### run the 6 methods ###########################
    for (itr in 1:numtimes) {
      input         <- setlakedefault(c(itr,itr),option)
      setup         <- lakesetup(input)
      plotout[itr,] <- setup$z
    }
    
    # now plot
    if (option==0) {
      figcount <- 13
    } else {
      figcount <- 14
    }
    cairo_pdf(paste("FigureA",figcount,"_",option,".pdf", sep=""), width=2.25, height=2.25)
    ylabel          <- 'Observed phosphorus [-]'
    xlabel          <- 'Time [years]'
    xaxisvals       <- seq(1,input$tend,1)
    truthcol        <- rgb(0,0,1)
    obscol          <- rgb(0.7,0.7,0.7,0.5)
    # start plotting
    par(mar=c(2.3,1.95,0.25,0.25), ps=9,mgp=c(1.2,0.5,0))
    if (option==0) {
      ylimvals <- c(0,0.5) 
    } else { 
      ylimvals <- c(0,2.8) 
    }
    plot(xaxisvals, plotout[1,],xlim=c(0,101), ylim=ylimvals, 
         xlab=xlabel, ylab=ylabel,type='l',col=rgb(0.4,0.4,0.4),
         lwd=2,cex.lab=1.0,cex.axis=0.8,tck=-0.05)
    for (i in 2:numtimes) {
      lines(xaxisvals, plotout[i,],col=obscol)
    }
    lines(xaxisvals, plotout[1,],lwd=2,col=rgb(0.4,0.4,0.4))
    # add the truth
    lines(xaxisvals, setup$xtrue[1,],col=truthcol)
    # add legend    
    if (option==0) {
      legend(x=-0.2,y=0.53,c('Observations','Truth'),
             col=c(obscol,truthcol), lty=c(1,1), bg=rgb(1,1,1,0), 
             y.intersp=0.7,x.intersp=0.7,cex=0.8,box.col=rgb(1,1,1,0))
      } else {
        legend(x=-0.2,y=2.9,c('Observations','Truth'),
           col=c(obscol,truthcol), lty=c(1,1), bg=rgb(1,1,1,0), 
           y.intersp=0.7,x.intersp=0.7,cex=0.8,box.col=rgb(1,1,1,0))
      }
    dev.off()    
  }