#' @name paperplot_Sup3
#' @aliases paperplot_Sup3
#' @title Runs the code for the supplementary figures in the publication.
#' @description This function runs the code for plotting number of usable ensembles/particles/samplse as a function of time.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot_Sup3 <- 
  function() {   
    label          <- c('EnsembleK','Particle','Precalib')    
    cols           <- c(rgb(0,1,0), rgb(0,1,1), rgb(0.8,0.8,0.8))    
    pchvals        <- c(1,4,5)
    numplts        <- 2
    numlines       <- 3
    # start plot    
    cairo_pdf(paste("FigureA15.pdf", sep=""), width=6, height=6)
    par(family="sans",cex=1.5,mar=c(3.5,3.5,1.5,0.5)+0.1,mgp = c(2.2,1.2,0),bg = NA)        
    mat <- matrix(seq(1,numplts,length=numplts),numplts,1,byrow=TRUE)
    layout(mat,widths=c(rep(1,numplts)),heights=c(rep(1,numplts)))
    for (option in 0:1) {      
      # initialize setup
      input          <- setlakedefault(c(1,1),option)
      setup          <- lakesetup(input)
      N              <- length(seq(input$tbeg,input$tend,input$timestep))
      xrange         <- c(1,seq(input$tbeg,input$tend,input$timestep))
      if (option==0) {
        load('paperplot2a.RData')
        # estimate depletion for the ensemble Kalman filter
        num_ensembles        <- array(NA,c(N+1,1))
        num_ensembles[1]     <- input$n_ens
        for (i in 1:N) {
          j                  <- i+1
          num_ensembles[j]   <- dim(filterout$ensemblekalmanout$x_enkfall[[i]])[1]  
        }
        # estimate depletion for the particle filter
        num_particles        <- array(NA,c(N+1,1))
        num_particles[1]     <- input$n_par
        for (i in 1:N) {
          j                  <- i+1
          num_particles[j]   <- dim(filterout$particleout$x_parall[[i]])[1]  
        }
        # estimate depletion for precalibration
        num_precalib         <- array(NA,c(N+1,1))
        num_precalib[1]      <- input$n_precalib
        for (i in 1:N) {
          j                  <- i+1
          num_precalib[j]    <- dim(filterout$precalib$xpar[[i]])[2]
        }    
        yrange <- range(c(num_ensembles,num_particles,num_precalib))
        for (i in 1:numlines) {
          if (i==1) {
            plot(xrange,rep(1,length(xrange)),col=rgb(1,1,1),axes=FALSE,ylim=yrange,
                 ylab="Ensembles/particles remaining [-]",xlab="Time [years]",xlim=c(1,100))
            axis(side=1,at=c(1,seq(10,100,by=10)))
            axis(side=2,at=seq(yrange[1],yrange[2],by=2500))
            mtext('a) Low emissions',side=3,at=15)
          }
          if (i==1) {
            points(xrange,num_ensembles,col=cols[i],pch=pchvals[i])      
            lines(xrange,num_ensembles,col=cols[i],pch=pchvals[i])      
          } else if (i==2) {
            points(xrange,num_particles,col=cols[i],pch=pchvals[i])      
            lines(xrange,num_particles,col=cols[i],pch=pchvals[i])      
          } else {
            points(xrange,num_precalib,col=cols[i],pch=pchvals[i])      
            lines(xrange,num_precalib,col=cols[i],pch=pchvals[i])      
          }
        }
      } else {
        load('paperplot2b.RData')
        # estimate depletion for the ensemble Kalman filter
        num_ensembles        <- array(NA,c(N+1,1))
        num_ensembles[1]     <- input$n_ens
        for (i in 1:N) {
          j                  <- i+1
          num_ensembles[j]   <- dim(filterout$ensemblekalmanout$x_enkfall[[i]])[1]  
        }
        # estimate depletion for the particle filter
        num_particles        <- array(NA,c(N+1,1))
        num_particles[1]     <- input$n_par
        for (i in 1:N) {
          j                  <- i+1
          num_particles[j]   <- dim(filterout$particleout$x_parall[[i]])[1]  
        }
        # estimate depletion for precalibration
        num_precalib         <- array(NA,c(N+1,1))
        num_precalib[1]      <- input$n_precalib
        for (i in 1:N) {
          j                  <- i+1
          num_precalib[j]    <- dim(filterout$precalib$xpar[[i]])[2]
        }       
        yrange <- range(c(num_ensembles,num_particles,num_precalib))
        for (i in 1:numlines) {
          if (i==1) {
            plot(xrange,rep(1,length(xrange)),col=rgb(1,1,1),axes=FALSE,
                 ylab="Ensembles/particles remaining [-]",xlab="Time [years]",xlim=c(1,100),ylim=yrange)
            axis(side=1,at=c(1,seq(10,100,by=10)))
            axis(side=2,at=seq(yrange[1],yrange[2],by=2500))
            legend(60,32000,label,horiz=FALSE,box.col=rgb(0,0,0),bg=rgb(1,1,1,0),y.intersp=1,x.intersp=0.5,
                   lwd=c(1,1,1),lty=c(1,1,1),pch=pchvals,col=cols,pt.bg=cols,pt.cex=c(2,2,2),cex=1.1)     
            mtext('b) High emissions',side=3,at=15)
          }
          if (i==1) {
            points(xrange,num_ensembles,col=cols[i],pch=pchvals[i])      
            lines(xrange,num_ensembles,col=cols[i],pch=pchvals[i])      
          } else if (i==2) {
            points(xrange,num_particles,col=cols[i],pch=pchvals[i])      
            lines(xrange,num_particles,col=cols[i],pch=pchvals[i])      
          } else {
            points(xrange,num_precalib,col=cols[i],pch=pchvals[i])      
            lines(xrange,num_precalib,col=cols[i],pch=pchvals[i])      
          }
        }
      }                
    }
    dev.off()    
  }