#' @name paperplot_Sup4
#' @aliases paperplot_Sup4
#' @title Runs the code for the supplementary figures in the publication.
#' @description This function runs the code for generating the MCMC performance plot.
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @export

paperplot_Sup4 <- 
  function(option=1) {    
    # load data from Figure 6
    load("paperplot6_10all.RData")    
    mcmcout2use    <- mcmcout2
    mcmcout2use    <- mcmcout2use[1,]
    rm(list=setdiff(ls(), "mcmcout2use"))
    mcmcout        <- mcmcout2use
    rm(mcmcout2use)
    # assign function handles
    input          <- setlakedefault(c(1,1),option)
    logposteriormod<- input$logposteriormod   
    model_handle   <- input$model_handle
    qtype          <- input$qtype
    seed           <- input$seed
    Bmat           <- input$Bmat
    uinput         <- input$uinput
    numstates      <- input$numstates
    numpars        <- input$numpars
    numbeg         <- numstates+1    
    numvars        <- numpars+numstates                # size of the state variable vector: phosphorus, b, and q, parameters 
    Qmod           <- input$Qmod
    qtype          <- input$qtype
    Qstates        <- getQ(Qmod[1:numstates], numstates)
    input$nummcmc  <- 25000
    setup          <- lakesetup(input)
    observations   <- setup$z 
    #estimate the sizes and assign arrays
    N              <- length(seq(1,input$tend))        # total number of time steps to simulate      
    # run the mcmc for time intervals of 10 years from year 1 to year 100
    timeperiods    <- c(1,seq(10,100,by=10))
    numtimes       <- length(timeperiods)    
    # now identify the MCMC predicted state and parameters for learning until time periods specified by 'timeperiods'    
    numchain       <- dim(mcmcout$outall[[1]])[2]                         # total number of chain (last 50%) for (j in 1:numchain) {
    chainout       <- array(NA,dim=c(numvars,numchain,numtimes))
    for (time in 1:numtimes) {      
      timerange    <- seq(input$tbeg,timeperiods[time],by=input$timestep) # define the timerange to run the model
      N            <- length(timerange)                                   # total number of time steps            
      for (j in 1:numchain) {
        temp         <- array(NA,dim=c(numvars,N))    
        temp[,1]     <- mcmcout$outall[[timeperiods[time]]][,j]
        for (i in 1:N) {          
          if (i==1) {
            temp[1:numstates,i]      <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+i,temp[1:numstates,i],temp[numbeg:numvars,i],timestep,i)+Bmat*uinput[i]          
          } else {
            temp[1:numstates,i]      <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+i,temp[1:numstates,i-1],temp[numbeg:numvars,i-1],timestep,i)+Bmat*uinput[i]        
            temp[numbeg:numvars,i]   <- temp[numbeg:numvars,i-1]
          }
        }
        chainout[,j,time] <- temp[,N]
        rm(temp)        
      }
      print(time)
    }    
    mcmcout$chainout <- chainout
    # save output in a single list for plotting later
    save(mcmcout, file = 'paperplot_Sup17.RData')  
    # now plot, use 1a for the no-flipping case, and 1b for the flipping case
    cairo_pdf(paste("FigureA17.pdf", sep=""), width=6, height=6)
    # define labels for plotting
    labels      <- input$labels
    # default colors
    truthcol       <- rgb(0,0,1)
    obscol         <- rgb(0,0,1,0.5)
    # define variables
    truth        <- setup$xtrue
    observations <- setup$z
    numobs       <- 1
    # start plot
    par(family="sans",cex=1.5,bg=NA)    
    # create the plot matrix
    par(mfcol=c(1,numvars))
    # specify y axis
    for (plt in 1:numvars) {
      # set x vals
      if (plt==1) {
        xrange  <- c(0.0,3.0)
        ygap    <- 10
      } else if (plt==2) {
        xrange  <- c(0.0, 1.0)
      } else if (plt==3) {
        xrange  <- c(-0.0,12.0)
      }
      ytot   <- ygap*numtimes
      yshift <- ygap*seq(0,numtimes-1)
      # create the plot
      par(mgp=c(1.5,0.8,0),mar=c(3.0,3.0,0.5,0.5)+0.1)        
      if (plt==1) {
        plot(xrange,rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",xlim=xrange,
             ylim=c(0,ytot),xlab=paste(labels[plt]," [units]"),ylab="PDF [-]",main=" ",
             cex.lab=1.5,frame.plot=FALSE,axes=FALSE)
        axis(1,pos=0,at=seq(xrange[1],xrange[2],by=0.50))        
        # insert y label
        mtext('Years ->',side=2,line=1,at=0,cex=1.0)
      } else if (plt==2) {
        plot(xrange,rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",xlim=xrange,
             ylim=c(0,ytot),xlab=paste(labels[plt]," [units]"),ylab=NA,main=" ",
             cex.lab=1.5,frame.plot=FALSE,axes=FALSE)        
        axis(1,pos=0,at=seq(0,1,by=0.5))
      } else if (plt==3) {
        plot(xrange,rep(0,length=length(xrange)),col="white",lwd=1,lty=1,type="l",xlim=xrange,
             ylim=c(0,ytot),xlab=paste(labels[plt]," [units]"),ylab=NA,main=" ",
             cex.lab=1.5,frame.plot=FALSE,axes=FALSE)        
        axis(1,pos=0,at=seq(-2,12,by=4))
      }     
      if (plt==1) {
        scalefac <- c(1,5,10,1,1,1,5,1,4,4,4)
      } else if (plt==2) {
        scalefac <- c(1,1,1,1,1,1,40,40,40,40,40)
      } else {
        scalefac <- c(2e-4,2e-4,2e-4,2e-4,3e-4,5e-2,5,1,5,1,15)
      }
      for (time in 1:numtimes) {
        if (sd(mcmcout$chainout[plt,,time])!=0) {
          mcmcval      <- density(mcmcout$chainout[plt,,time],bw=scalefac[time]*sd(mcmcout$chainout[plt,,time]),na.rm=TRUE)
          if (scalefac[time]==1) {
            lines(mcmcval$x,mcmcval$y+yshift[time],lwd=1,col=rgb(0,0,0),lty=1)        
          } else {
            lines(mcmcval$x,mcmcval$y+yshift[time],col=rgb(0.5,0.5,0.5,0.5),lwd=2,lty=1)        
          }
        } else {
          timenext <- time+1
          xvalue   <- unique(mcmcout$chainout[plt,,time])
          lines(c(xvalue,xvalue),c(yshift[time],yshift[timenext]),lwd=2,col=rgb(0,0,0),lty=1)
        }
        lines(c(xrange),c(yshift[time],yshift[time]),col=rgb(0.7,0.7,0.7,0.5),lwd=1)
        # plot the truth 
        lines(c(truth[plt,timeperiods[time]],truth[plt,timeperiods[time]]),c(yshift[time],yshift[time]+ygap),col=truthcol,lwd=1,lty=1)        
        if (plt==1) {
          # insert y labels       
          mtext(timeperiods[time],side=2,line=0,at=yshift[time]+ygap/2,cex=1.0)
        }
      }      
      # plot the observations
      if (numobs>=plt) {
        if (numobs == 1) {
          points(observations[timeperiods],yshift,col=obscol,type="p")
        } else {
          points(observations[plt,timeperiods],yshift,col=obscol,type="p")
        }
      }
      # plot the legend
      if (plt==1) {                 
        legend(0.15,15,c('MCMC',"Truth","Observations"),
               horiz=FALSE, box.col=NA,y.intersp=1,x.intersp=1.0,lwd=c(1,1,0),
               lty=c(1,1,0),pch=c(26,26,1),pt.lwd=c(1,1,3),col=c("black",truthcol,obscol),
               cex=1.3,ncol=1)
      }        
    }             
    dev.off()
  }