#' @name pcritestimator
#' @aliases pcritestimator
#' @title Estimate the critical phosphorus threshold value
#' @description This function returns the critical phosphorus threshold value for a given set of b and q parameters
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value

pcritestimator <- 
  function(parvec) {    
    # estimate whether parvec has just one combination of b and q or several
    pcrit <- NaN
    if (is.null(dim(parvec))==TRUE) {
      if (is.nan(parvec[1])==TRUE || (is.nan(parvec[2])==TRUE) || (parvec[1]<0) || (parvec[2]<0) || (parvec[1]>1)) {
        pcrit <- NaN
      } else {
        temp     <- NaN
        try(temp <- uniroot(function(y) y^parvec[2]/(1+y^parvec[2]) - parvec[1]*y, c(0.0001, 1.6))$root)
        pcrit    <- temp 
      }
    } else {
      numsets <- dim(parvec)[1]
      pcrit   <- rep(NaN, numsets)          
      for (i in 1:numsets) {        
        temp    <- NaN
        # check for NaNs
        if ((is.nan(parvec[i,1])==TRUE) || (is.nan(parvec[i,2])==TRUE) || (is.na(parvec[i,1])==TRUE) || (is.na(parvec[i,2])==TRUE) || (parvec[i,1]<0) || (parvec[2]<0) || (parvec[1]>1)) {
          temp     <- NaN
        } else {
          try(temp <- uniroot(function(y) y^parvec[i,2]/(1+y^parvec[i,2]) - parvec[i,1]*y, c(0.0001, 1.6))$root)
        }
        pcrit[i] <- temp
        print(i)
      }
    }
    return(pcrit)
  }