#' @name probeutro
#' @aliases probeutro
#' @title Assess the odds ratio of tip vs. no tip for a given projection
#' @description This function assesses the ability of a filter to detect tipping points
#' @param object  A matrix or vector with initial condition(s)
#' @param truth The true values of the variables (in case of a synthetic experiment or the demostration example)
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A list with values of both criteria
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso \code{\link{tippingpredict}} \code{\link{paperplot4}}
#' @export

probeutro <-
  function(object,input,setup,tipthres,numsim)
  {        
    # set output array size
    flipcnt          <- array(NA,c(numsim,1))
    # remove nans
    nanind           <- which(is.nan(object)==TRUE)
    nonanind         <- setdiff(seq(1,length(object)),nanind)
    object           <- object[nonanind]
    numsim           <- length(object)
    # run the model numsim times
    for (itr in 1 : numsim) {      
      # assess whether the simulation flips or not
      if (object[itr] > tipthres) {
        flipcnt[itr]      <- 1
      } else {
        flipcnt[itr]      <- 0
      }
    }
    # assess the probability of being in a eutrophic state
    probeutro   <- length(which(flipcnt==1))/(length(which(flipcnt==0))+length(which(flipcnt==1)))
    # print the ratio of NaNs    
    print(paste(length(nanind), " NaNs encountered", sep=""))
    return(probeutro)      
  }