#' @name resample
#' @aliases resample
#' @title Implement the resampling scheme for the particle filter
#' @description This function implements three resampling scheme for the particle filter: stratified, systematic, and Ripley's
#' @param x                   An array of particles
#' @param w                   A vector of particle weights
#' @param resamplechoice      Choice of the resampling scheme, 1 for stratified, 2 for systematic, and 3 for Ripley's
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @return  A list with updated particles and associated weights
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note      %% add value
#' @seealso   dafilter.particle
#' @examples  %%  ~~any examples~~

resample <-
  function(x,w, resamplechoice) {    
    N  <- length(w)
    if (resamplechoice==1) {
      # stratified sampling
      u  <- (seq(0,N-1)+runif(N,0,1))/N
    } else if (resamplechoice==2) {
      # systematic sampling
      u  <- (seq(0,N-1)+runif(1,0,1))/N
    } else if (resamplechoice==3) {
      # Multinomial sampling with Ripley's method
      u  <- rev(cumprod(runif(N,0,1)^(1/seq(N,1,-1))))
    }    
    wc   <- cumsum(w)
    ind  <- array(0, dim=c(N,1))
    # generating resampling indices: if a particle has higher weight,
    # indices corresponding to its location are selected more often
    # in this way, high weight particlesa re sampled more frequently
    k = 1
    for (i in 1:N) {
      while ((wc[k] < u[i]) == TRUE) { k = k+ 1}
      ind[i] = k
    }
    # assigning the new particle vector based on the new indices
    if (is.null(dim(x))==TRUE) {
      x <- x[ind]
    } else {
      x <- x[,ind]
    }
    # now, the weights of all particles are equal- since we have more number of high weight particles
    w <- rep(1,N)/N
    #assigning the return variables
    returnval   <- list(w=w,x=x)
    returnval$x <- x
    returnval$w <- w
    return(returnval)
  }