#' @name setbasecasedefault
#' @aliases setbasecasedefault
#' @title Set the default values of the base case model parameters
#' @description This function sets the default values of the base case model parameters such as a, b, and initial values of x
#' @return  A list with the default values of the model
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note %% add value
#' @seealso \code{\link{nltssetup}}, \code{\link{f_nlts}}, \code{\link{h_nlts}}
#' @examples
#' #using the demostration example set up
#' input <- setnltsdefault()
#' @export

setbasecasedefault <- function(seed = NULL)
{
  #initialize default
  default                    <- list()
  #set the name if this initialization
  default$name               <- 'linearmodel'
  #choose the filters to try: 1="on", 0="off"
  default$runKF              <- 1
  default$runEKF             <- 1
  default$runEnKF            <- 1
  default$runPF              <- 1
  default$prior              <- 0 # 0 for normal prior and 1 for lognormal
  # choose measurement variance
  # in reality, this would be set by the measurement device  
  default$R                  <- 10
  # choose model stochastic noise
  # note: particle filter will collapse to a single point very quickly in case of small process noise 
  # one can set it to some small non zero by multiplying Qmod with 1
  default$Qmod               <- c(0.001, 0.001)*0
  default$qtype              <- 0 # 0 for normal and 1 for lognormal noise
  # choose beginging and end time for the simulation (years)
  default$tbeg               <- 1
  default$tend               <- 100  
  #choose time interval
  default$timestep           <- 1
  #choose the seed type
  if (is.null(seed) == TRUE) {
    default$seed              <- c(as.numeric(Sys.time()),as.numeric(Sys.time()))
  } else if (length(seed)==1) {
    default$seed             <- c(seed, as.numeric(Sys.time()))
  } else {
    default$seed             <- seed
  }
  #setting up filters
  default$n_ens              <- 1000
  default$resamplechoice     <- 2
  default$n_par              <- default$n_ens   # for both filters to be comparable
  default$plottype           <- 1
  # setting up the truth, first guess, initial esitmate of covariance
  default$xtruestate         <- array(0.00  ,dim=c(1,1))        # true phosphorus in the lake (-), b parameter (-), q parameter (-)
  default$xguess             <- array(0.50  ,dim=c(1,1))        # first guess for the state variables and parameters
  default$Pxguess            <- 1.00                            # initial estimate of state and parameter error covariance
  default$xtruepars          <- array(3.00  ,dim=c(1,1))        # true phosphorus in the lake (-), b parameter (-), q parameter (-)
  default$xguesspars         <- array(2.50 ,dim=c(1,1))        # first guess for the state variables and parameters
  default$Pxguesspars        <- 1.00                            # initial estimate of state and parameter error covariance
  # define the number of iterations for performance calculations
  default$numtimes           <- 10
  # define the number of samples for precalibration
  default$n_precalib         <- default$n_ens
  # define the number of times to run mcmc
  default$nummcmc            <- default$n_ens
  # read in the default anthropogenic pollution load time series
  default$uinput             <- rep(0,  length(seq(0,default$tend, by = default$timestep)))
  # set the default mulitplier on input
  default$Bmat               <- 0  
  # set the flag on CPU time: 1 (display for each filter), 0  (do not display)
  default$timeflag           <- 1
  default$numpars            <- 1
  default$numstates          <- 1
  default$model_handle       <- f_basecasestate
  default$measurement_handle <- h_basecase
  default$param_handle       <- f_basecasepars
  default$parammeas_handle   <- hmat_basecasepars
  default$logposteriormod    <- logposteriorbase
  # set labels for plotting
  default$labels                <- c('X [-]', 'b parameter [-]')  
  return(default)
}
