#' @name summary.filter
#' @aliases summary.filter
#' @title Estimate he summary statistics of the output filter class
#' @description This function estimates the first and last values of the mean of the forecasted variables, and the standard deviation from the covaraince matrices of the output filter class
#' @param object                 An output of filter class
#' @details %%  ~~ If necessary, more details than the dcription above ~~
#' @return  An object of the filter summary class
#' @references %% ~~No references~~
#' @author Riddhi Singh
#' @note      %% add value
#' @seealso   dafilter.default
#' @seealso   print.summary.filter
#' @examples
#' #using the demostration example set up
#' model_handle       <- f_model
#' measurement_handle <- h_meas
#' input              <- setdefault()
#' setup              <- problemsetup(input)
#' kalmanout          <- dafilter(model_handle, measurement_handle, setup, input)
#' kalmansummary      <- summary(kalmanout)
#' print.summary.filter(kalmansummary)
#' @export

summary.filter <-
  function(object)
  {
    sz          <- dim(object$x_mean)
    if (is.null(sz) == TRUE) {
      N            <- length(object$x_mean)
      numvars      <- 1
    } else {
      N            <- max(sz)
      numvars      <- min(sz)
    }

    standarddev    <- array(0, dim=c(N, numvars))
    for (i in 1:numvars) {
      standarddev[,i]  <- sqrt(abs(object$x_cov[i,i,]))
    }

    stddevfirst <- standarddev[1,]
    stddevlast  <- standarddev[N,]
    firstval    <- object$x_mean[,1]
    lastval     <- object$x_mean[,N]
    tab         <- cbind(stddevfirst = stddevfirst, stddevlast = stddevlast, firstval = firstval, lastval = lastval)
    filtersum   <- list(call=object$call, boundingvals = tab, std = standarddev)

    #define the summary class
    class(filtersum) = "summary.filter"
    filtersum
  }
