#' @name tippingpredict
#' @aliases tippingpredict 
#' @title Project the future based on the output of a learning method until a given time
#' @description This function runs the lake model using output from the either of the six methods
#' @param object  A matrix or vector with initial condition(s)
#' @param truth The true values of the variables (in case of a synthetic experiment or the demostration example)
#' @details %%  ~~ If necessary, more details than the description above ~~
#' @return  A list with values of both criteria
#' @author Riddhi Singh
#' @note  %% add value
#' @seealso \code{\link{kalmanfitler}} \code{\link{extendedkalmanfitler}} \code{\link{ensemblekalmanfitler}} \code{\link{particlefitler}}

tippingpredict <-
  function(object,input,setup,calyr)
  {        
    #estimate the size
    model_handle  <- input$model_handle
    timestep      <- input$timestep
    Qmod          <- input$Qmod
    numstates     <- input$numstates
    numpars       <- input$numpars
    qtype         <- input$qtype
    seed          <- input$seed
    Bmat          <- input$Bmat
    uinput        <- input$uinput
    numbeg        <- numstates+1
    numvars       <- numpars+numstates
    Qstates       <- getQ(Qmod[1:numstates], numstates)
    Qpars         <- getQ(Qmod[numbeg:numvars], numpars)
    # initialize some year counts    
    siminit       <- calyr + 1           
    # set the prediction times
    timerange     <- seq(siminit,input$tend,by=timestep)            # define the timerange to run the model
    N             <- length(timerange)                              # total number of time steps        
    # set output array size
    numsims       <- dim(object)[3]
    outobj        <- array(NA,c(numvars,N,numsims))
    returnval     <- array(NA,c(numvars,numsims))
    # loop over simulations
    for (sims in 1:numsims) {
      if (calyr==0) {
        caltemp            <- calyr+1
        outobj[,1,sims]    <- object[,caltemp,sims]  
      } else {
        outobj[,1,sims]    <- object[,calyr,sims]  
      }
      # now loop over time
      for (simt in siminit:input$tend) {        
        tempi                       <- simt - calyr 
        if (simt == siminit) {
          outobj[1:numstates,tempi,sims]    <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+tempi,outobj[1:numstates,tempi,sims],outobj[numbeg:numvars,tempi,sims],timestep,tempi)+Bmat*uinput[simt]        
        } else {
          outobj[1:numstates,tempi,sims]    <- addprocessnoise(model_handle,Qstates,qtype,seed[2]+tempi,outobj[1:numstates,tempi-1,sims],outobj[numbeg:numvars,tempi-1,sims],timestep,tempi)+Bmat*uinput[simt]        
          outobj[numbeg:numvars,tempi,sims] <- outobj[numbeg:numvars,tempi-1,sims]            
        }      
      }      
      returnval[,sims] <- outobj[,N,sims]
    }
    return(returnval)      
  }